/* 
 * Copyright (c) 2012 Qualcomm Atheros, Inc.. 
 * All Rights Reserved. 
 * Qualcomm Atheros Confidential and Proprietary. 
 */

#include "common.h"
#include "spiflash.h"

#define SPI_FLASH_BASE      0xb9000000
#define SPI_FLASH_SIZE      0x400000

#if 0
#define IMAGE_A_COMP_OFFSET  0x300000
#define IMAGE_A_OFFSET       0x80000
#define IMAGE_B_OFFSET       0x200000
#define IMAGE_B_COMP_OFFSET  0x380000
#define CONTROL_BLOCK_OFFSET 0x380000
#define BOARD_INFO_OFFSET    0x384000
#define CONFIG_FILE_OFFSET   0x386000
#else
#define IMAGE_A_COMP_OFFSET  0x40000
#define IMAGE_A_OFFSET       0x1c0000
#define IMAGE_B_OFFSET       0x200000
#define IMAGE_B_COMP_OFFSET  0x380000
#define CONTROL_BLOCK_OFFSET 0x3c000
#define BOARD_INFO_OFFSET    0x384000
#define CONFIG_FILE_OFFSET   0x386000
#endif

#if 0
#define IMAGE_A_OFFSET       0x40000
#define IMAGE_B_OFFSET       0x100000
#define IMAGE_RUNNING_OFFSET 0x1c0000
#define CONTROL_BLOCK_OFFSET 0x3c000
#define BOARD_INFO_OFFSET    0x3c0000
#define CONFIG_FILE_OFFSET   0x3c4000
#endif


int soc_bus_freq_get(void);
void serial_baudrate_set(int baudrate);
void main_end(void);

#define ZM_USER_BREAK_KEY       0x03    /* ^C - Control C */
#define ZM_USER_BREAK_CHG_BAUD_KEY       0x04   /* ^D - Control D */
//#define ZM_USER_BREAK_KEY       0x03       /* ^C - Control C */

#define ZM_USER_BREAK_BURN_LOADER_KEY    0x03   /* ^C - Control C */
#define ZM_USER_BREAK_BURN_CTRL_KEY      0x05   /* ^E - Control E */
#define ZM_USER_BREAK_BURN_OEM_KEY       0x06   /* ^F - Control F */
#define ZM_USER_BREAK_BURN_IMAGE_KEY     0x07   /* ^G - Control G */
#define ZM_USER_BREAK_INVALID_CONF_KEY   0x02   /* ^B - Control B */

extern uint32_t image_flash_start;
extern uint32_t image_flash_size;


static int zmodem_user_break(int delay_secs)
{
    int abort = 0, abort_key = 0;

    printf("Press ctrl+c to burn loader image\n");
#if 0
    printf("Press ctrl+e to burn control info\n");
    printf("Press ctrl+f to burn oem info\n");
    printf("Press ctrl+g to burn compressed imageB\n");
    printf("Press ctrl+i to burn loader image\n");
    printf("Press ctrl+h to burn compressed imageA\n");
    printf("Press ctrl+b to invalidate confile file\n");
#endif

    while ((delay_secs > 0) && (!abort))
    {
        int i;

        --delay_secs;

        /* delay 100 * 10ms */
        for (i = 0; !abort && i < 100; ++i)
        {
            if (serial_tstc())
            {
                abort_key = serial_getc();
                if (abort_key == ZM_USER_BREAK_BURN_LOADER_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    image_flash_start = 0;
                    image_flash_size = 0x20000;
                    goto OUT;
                }
#if 0
                else if (abort_key == ZM_USER_BREAK_BURN_CTRL_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    image_flash_start = CONTROL_BLOCK_OFFSET;
                    image_flash_size = 0x4000;
                    goto OUT;
                }
                else if (abort_key == ZM_USER_BREAK_BURN_OEM_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    image_flash_start = BOARD_INFO_OFFSET;
                    image_flash_size = 0x100;
                    goto OUT;
                }
                else if (abort_key == ZM_USER_BREAK_BURN_IMAGE_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    image_flash_start = IMAGE_B_COMP_OFFSET;
                    image_flash_size = 0x80000;
                    //image_flash_start = 0x10000;
                    //image_flash_size = 0x10000;          
                    goto OUT;
                }
                else if (abort_key == 0x9)
                {
                    abort = 1;
                    delay_secs = 0;
                    //image_flash_start = IMAGE_A_OFFSET;
                    //image_flash_size = 0x180000;   
                    image_flash_start = 0x4000;
                    image_flash_size = 0x10000;
                    goto OUT;
                }
                else if (abort_key == ZM_USER_BREAK_INVALID_CONF_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    image_flash_start = CONFIG_FILE_OFFSET;
                    image_flash_size = MUSIC_SPI_SECTOR_SIZE;
                    music_spi_init();
                    music_spi_sector_erase(image_flash_start);
                    printf("complete invalidate config file!\n");
                    while (1);
                    goto OUT;
                }
                else if (abort_key == 0x8)
                {
                    abort = 1;
                    delay_secs = 0;
                    image_flash_start = IMAGE_A_COMP_OFFSET;
                    image_flash_size = 0x80000;
                    goto OUT;
                }
#endif
                abort_key = 0;
            }
            udelay(10000);
        }

        printf("\b\b\b%2d ", delay_secs);
    }

  OUT:
    printf("\n");

    return abort_key;
}

static int zmodem_get_speed(int delay_secs)
{
    int abort = 0, abort_key = 0;

    //printf("Press ctrl+d to select 19200 bps baud speed\n");
    //printf("Press ctrl+c to into zmodem by using 9600 bps baud speed: %2d ", delay_secs);
    printf("%2d ", delay_secs);

    while ((delay_secs > 0) && (!abort))
    {
        int i;

        --delay_secs;

        /* delay 100 * 10ms */
        for (i = 0; !abort && i < 100; ++i)
        {
            if (serial_tstc())
            {
                abort_key = serial_getc();
                if (abort_key == ZM_USER_BREAK_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    goto OUT;
                }
                else if (abort_key == ZM_USER_BREAK_CHG_BAUD_KEY)
                {
                    abort = 1;
                    delay_secs = 0;
                    goto OUT;
                }
                abort_key = 0;
            }
            udelay(10000);
        }

        printf("\b\b\b%2d ", delay_secs);
    }

  OUT:
    printf("\n");

    return abort_key;
}


int main(void)
{
    int abort_key = 0;

    serial_init();
    printf("serial_init done!\n");

    printf("[cpu]: %dMHZ\n", (soc_bus_freq_get() * 2 / 1000000));


    if ((abort_key = zmodem_user_break(20)))
    {
        printf("Press ctrl+d to select 19200 bps baud speed\n");
        printf("Press ctrl+c to into zmodem by using 9600 bps baud speed: ");
        abort_key = zmodem_get_speed(20);
        if (abort_key == ZM_USER_BREAK_CHG_BAUD_KEY)
        {
            printf("please change serial baudrate to 19200 bps in 30 seconds\n\n");
            serial_baudrate_set(19200);
            zmodem_get_speed(30);
        }
        printf("zmodem start\n");
        zmodem_main();

    }

    printf("you should not get this!\n");
    while (1);

#if 1
    unsigned long times = 0, addr, start = 0xa0000000, len = 10 * 1000 * 1000;
    while (1)
    {
        for (addr = start; addr < start + len; addr += 4)
        {
            *(volatile int *)addr = addr;

            if (*(volatile int *)addr == addr)
            {
                if (addr % 1000 == 0)
                {

                    printf("times:%x ddr scan [%x] pass\n", times, addr);
                }
            }
            else
            {
                printf("ddr fail at addr:%x expect:%x actually:%x \n", addr, addr, *(int *)addr);
            }
        }
        if (times > 0xfffffff0)
        {
            times = 1;
        }
        times++;
    }
#endif


    main_end();

    return 0;
}
