#ifndef _MUSIC_PHY_H_
#define _MUSIC_PHY_H_

#ifdef __cplusplus
extern "C" {
#endif    /* __cplusplus */


#define SWITCH_REG_BASE 0xb8800000

#define REG_MDIO_CTRL  0x15004
#define MDIO_CTRL_MDIO_BUSY_MASK    	0x80000000
#define MDIO_CTRL_MDIO_BUSY_OFFSET  	31
#define MDIO_CTRL_PHY_SEL_MASK    		0x30000000
#define MDIO_CTRL_PHY_SEL_OFFSET    	28
#define MDIO_CTRL_MDIO_CMD_MASK	    	0x08000000
#define MDIO_CTRL_MDIO_CMD_OFFSET   	27
#define MDIO_CTRL_MDIO_NO_SUP_PRE_MASK	 0x04000000
#define MDIO_CTRL_MDIO_NO_SUP_PRE_OFFSET 26
#define MDIO_CTRL_PHY_ADDR_MASK    		0x03e00000
#define MDIO_CTRL_PHY_ADDR_OFFSET    	21
#define MDIO_CTRL_REG_ADDR_MASK    		0x001f0000
#define MDIO_CTRL_REG_ADDR_OFFSET  		16
#define MDIO_CTRL_MDIO_DATA_MASK    	0x0000ffff
#define MDIO_CTRL_MDIO_DATA_OFFSET    	0
#define MDIO_CMD_READ  1
#define MDIO_CMD_WRITE 0

#define PHY_ACCEL_MAX 	   3
#define PHY_ACCEL_PORT_MAX 8
#define PHY_TYPE_ACCEL_ID0 0x004D
#define PHY_TYPE_ACCEL_A0_ID1 0xD081
#define PHY_TYPE_ACCEL_B0_ID1 0xD082
#define PHY_REG_PHY_ID0        0x02
#define PHY_REG_PHY_ID1        0x03
#define MUSIC_PHY_IS_ACCEL_A0(phy_id0, phy_id1) ((phy_id0 == PHY_TYPE_ACCEL_ID0)&& \
												(phy_id1 == PHY_TYPE_ACCEL_A0_ID1))
#define MUSIC_PHY_IS_ACCEL_B0(phy_id0, phy_id1) ((phy_id0 == PHY_TYPE_ACCEL_ID0)&& \
												(phy_id1 == PHY_TYPE_ACCEL_B0_ID1))

uint32_t switch_reg_rd(uint32_t reg);
int32_t switch_reg_wr(uint32_t reg, uint32_t val);

#define switch_mdio_phy_rd(phy_id, port_id, reg, val) \
		_switch_mdio_phy_rd(phy_id, port_id, reg, val, \
							switch_reg_rd, switch_reg_wr)

#define switch_mdio_phy_wr(phy_id, port_id, reg, val) \
		_switch_mdio_phy_wr(phy_id, port_id, reg, val, \
							switch_reg_rd, switch_reg_wr)

#ifdef MUSIC_B2B_SUPPORT
uint32_t mdio_switch_reg_rd(uint32_t addr);
int32_t mdio_switch_reg_wr(uint32_t addr, uint32_t data);

#define switch_slave_mdio_phy_rd(phy_id, port_id, reg, val) \
		_switch_mdio_phy_rd(phy_id, port_id, reg, val, \
							mdio_switch_reg_rd, mdio_switch_reg_wr)

#define switch_slave_mdio_phy_wr(phy_id, port_id, reg, val) \
		_switch_mdio_phy_wr(phy_id, port_id, reg, val, \
							mdio_switch_reg_rd, mdio_switch_reg_wr)
#endif

#ifdef __cplusplus
}
#endif    /* __cplusplus */

#endif                          //_MUSIC_PHY_H_
