/* * Copyright (c) 2012 Qualcomm Atheros, Inc.
 * All Rights Reserved. 
 *Qualcomm Atheros Confidential and Proprietary. 
 *Notifications and licenses are retained for attribution purposes only */
/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards.                            */
/*  Released to the Public Domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  romboot_zmodem - core zmodem routines                            */
/*                                                                   */
/*********************************************************************/

#include <common.h>
#include <zmodem_crc32.h>
#include <zmodem_crcxm.h>
#include <zmodem.h>
#include "spiflash.h"


static int32_t romboot_errorno = 0;

static void zmodem_receive_file(ZMCORE * romboot_zmodem);
static void zmodem_receive_data(ZMCORE * romboot_zmodem);
static int32_t zmodem_pos_match(ZMCORE * romboot_zmodem);
static void zmodem_get_data(ZMCORE * romboot_zmodem);
static void zmodem_get_data16(ZMCORE * romboot_zmodem);
static void zmodem_get_data32(ZMCORE * romboot_zmodem);
static void zmodem_get_filedata(ZMCORE * romboot_zmodem);
static void zmodem_get_header(ZMCORE * romboot_zmodem);
static void zmodem_get_hex_header(ZMCORE * romboot_zmodem);
static void zmodem_get_binary_header(ZMCORE * romboot_zmodem);
static void zmodem_get_bin32_header(ZMCORE * romboot_zmodem);
static void zmodem_get_next_hexch(ZMCORE * romboot_zmodem);
static void zmodem_get_next_DLEch(ZMCORE * romboot_zmodem);
static void zmodem_get_next_ch(ZMCORE * romboot_zmodem);
static void zmodem_send_header(ZMCORE * romboot_zmodem);
static void zmodem_send_hex_header(ZMCORE * romboot_zmodem);
static void zmodem_send_bin_header(ZMCORE * romboot_zmodem);
static void zmodem_send_bin32_header(ZMCORE * romboot_zmodem);
static void zmodem_send_char(ZMCORE * romboot_zmodem);
static void zmodem_send_DLE_char(ZMCORE * romboot_zmodem);
static void zmodem_send_hex_char(ZMCORE * romboot_zmodem);
static void zmodem_send_ZRINIT(ZMCORE * romboot_zmodem);
static void zmodem_send_ZKIP(ZMCORE * romboot_zmodem);
static void zmodem_send_ZRPOS(ZMCORE * romboot_zmodem);
static void zmodem_send_ZFIN(ZMCORE * romboot_zmodem);
static void zmodem_get_OO(ZMCORE * romboot_zmodem);
extern void music_spi_write_page(uint32_t addr, uint8_t * data, int len);
extern void music_spi_sector_erase(uint32_t addr);
static uint32_t receive_count_16 = 0, begin_receive_file = 0;
#define IMAGE_FLASH_START   0x0000
#define IMAGE_FLASH_SIZE    0x4000
uint32_t image_flash_start = 0;
uint32_t image_flash_size = 0;

/* strictly we don't need to escape all of these characters.
   If the environment can send 0x7f we don't need to DLE
   escape it.  If the environment is 8-bit clean we don't
   need all the 8-bit escapes.  But we escape everything
   here by default to be on the safe side. */

extern void call_fw(int, const unsigned int);
#ifdef DONT_ESCAPE_7F
#define needsDLE2(x) (((x) == ZDLE) || ((x) == 0x10) || ((x) == 0x11) \
                     || ((x) == 0x13) )
#else
#define needsDLE2(x) (((x) == ZDLE) || ((x) == 0x10) || ((x) == 0x11) \
                     || ((x) == 0x7f) \
                     || ((x) == 0x13) )
#endif

#ifdef IS_8BIT_CLEAN
#define needsDLE(x) needsDLE2((x))
#else
#define needsDLE(x) needsDLE2((x) & ~0x80)
#endif

//#define RDEBUG

#define modem_register_good(x)  //printf("Zmodem work well\r\n")
#define modem_register_bad(x)   //printf("Zmodem error\r\n")

#ifdef ZMODEM_RAM_DEBUG
extern uint8_t *printf_buf;
extern int32_t printf_pos;
#endif

void romboot_zmodem_init(ZMCORE * romboot_zmodem)
{
    romboot_zmodem->main_buf = (uint8_t *) ZMODEM_RECEIVE_BASEADDR;
    romboot_zmodem->bufsize = ZMODEM_BUFFER_SIZE;
    romboot_zmodem->maxtx = ZMCORE_MAXTX;
    romboot_zmodem->buf_pos = romboot_zmodem->main_buf;
#ifdef ZMODEM_RAM_DEBUG
    printf_buf = (uint8_t *) 0xa0100000;
    printf_pos = 0;

    for (printf_pos = 0; printf_pos < 0x5000; printf_pos++)
    {
        printf_buf[printf_pos] = 0;
    }
    printf_pos = 0;
#endif
    return;
}

void romboot_zmodem_term(ZMCORE * romboot_zmodem)
{
    romboot_zmodem->main_buf = NULL;
    return;
}

int32_t romboot_zmodem_receive(ZMCORE * romboot_zmodem)
{
    int32_t quit = 0;
    int32_t tries = 0;

    romboot_zmodem->buf_top = romboot_zmodem->main_buf + romboot_zmodem->bufsize;
    //extModemClearInbound(romboot_zmodem);
    romboot_zmodem->skip = 0;

    while (ALLOK)
    {
        /* When the zmodem receive program starts, it immediately sends
         * a ZRINIT header to initiate zmodem file transfers */
        zmodem_send_ZRINIT(romboot_zmodem);
        quit = 0;
        tries = 0;

        while (ALLOK && !quit)
        {
            zmodem_get_header(romboot_zmodem);

            if (ALLOK)
            {
                /* ZFIN represent transfer finished */
                if (romboot_zmodem->header_type == ZFIN)
                {
                    /* send ZFIN to ack the ZFIN */
                    zmodem_send_ZFIN(romboot_zmodem);
                    modem_register_good(romboot_zmodem);
                    zmodem_get_OO(romboot_zmodem);
                    printf("complete flash burn!\n");
                    while (1);
                    //call_fw(0, ZMODEM_RECEIVE_BASEADDR);
                    printf("* * * * * *return from image* * * * * *\n");
                    quit = 1;
                    return 1;   //raoc
                }
                else if (romboot_zmodem->header_type == ZRQINIT)
                {
                    /* If the receiving program receives a ZRQINIT header,
                     * it resends the ZRINIT header. */
                    if (tries++ < 10)
                    {
                        zmodem_send_ZRINIT(romboot_zmodem);
                    }
                    else
                    {
                        zmodem_error_set(ZMODEM_INIT);
                    }
                }
                else if (romboot_zmodem->header_type == ZFILE)
                {
                    romboot_zmodem->skip = 0;
                    modem_register_good(romboot_zmodem);

                    zmodem_get_filedata(romboot_zmodem);

                    if (ALLOK)
                    {
                        zmodem_receive_file(romboot_zmodem);
                        quit = 1;
                    }
                    else if (zmodem_error_compare(ZMODEM_CRC32)
                             || zmodem_error_compare(ZMODEM_CRCXM) || zmodem_error_compare(ZMODEM_LONGSP))
                    {
                        if (tries++ < 10)
                        {
                            zmodem_error_flush();
                            zmodem_send_ZRINIT(romboot_zmodem);
                        }
                    }

                }
            }
            else
            {

                if (zmodem_error_compare(ZMODEM_TIMEOUT)
                    || zmodem_error_compare(ZMODEM_CRCXM)
                    || zmodem_error_compare(ZMODEM_CRC32) || zmodem_error_compare(ZMODEM_BADHEX))
                {
#ifdef RDEBUG
                    ZM_INFO("TIME OUT RETRY in zmodem receive\n");
#endif
                    if (tries++ < 10)
                    {
                        zmodem_error_flush();
                        zmodem_send_ZRINIT(romboot_zmodem);
                    }
                }
            }
        }
    }
    return 0;
}

static void zmodem_receive_file(ZMCORE * romboot_zmodem)
{
    int32_t quit = 0;
    int32_t tries = 0;

#ifdef RDEBUG
    ZM_INFO("in zmodem_receive_file\n");
#endif
    begin_receive_file = 1;
    romboot_zmodem->good_offset = 0;
    romboot_zmodem->skip = 0;

    if (!ALLOK)
    {
        return;
    }

    if (romboot_zmodem->skip)
    {
        zmodem_send_ZKIP(romboot_zmodem);
        return;
    }

    zmodem_send_ZRPOS(romboot_zmodem);

    while (ALLOK && !quit)
    {
        zmodem_get_header(romboot_zmodem);

        if (ALLOK)
        {

            if (romboot_zmodem->header_type == ZFILE)
            {

                if (tries++ < 10)
                {
                    zmodem_send_ZRPOS(romboot_zmodem);
                }
                else
                {
                    zmodem_error_set(ZMODEM_POS);
                }
            }
            else if (romboot_zmodem->header_type == ZDATA)
            {
                modem_register_good(romboot_zmodem);
                zmodem_receive_data(romboot_zmodem);
                quit = 1;
            }
            else if (romboot_zmodem->header_type == ZEOF)
            {

                if (zmodem_pos_match(romboot_zmodem))
                {
                    modem_register_good(romboot_zmodem);
                    quit = 1;
                }
                else
                {

                    if (tries++ < 10)
                    {
                        zmodem_send_ZRPOS(romboot_zmodem);
                    }
                    else
                    {
                        zmodem_error_set(ZMODEM_POS);
                    }
                }
            }
        }
        else
        {

            if (zmodem_error_compare(ZMODEM_TIMEOUT)
                || zmodem_error_compare(ZMODEM_CRCXM) || zmodem_error_compare(ZMODEM_CRC32))
            {

                if (tries++ < 10)
                {
                    zmodem_error_flush();
                    zmodem_send_ZRPOS(romboot_zmodem);
                }
            }
        }
    }

    return;
}

static void zmodem_receive_data(ZMCORE * romboot_zmodem)
{
    int32_t quit = 0;
    int32_t tries = 0;
    int32_t timeouts = 0;
    int32_t head_tries = 0;

    romboot_zmodem->moredata = 1;

    while (ALLOK && !quit)
    {

        if (romboot_zmodem->moredata)
        {
            zmodem_get_data(romboot_zmodem);

            if (ALLOK)
            {
                tries = 0;
                timeouts = 0;
            }
        }
        else
        {
            zmodem_get_header(romboot_zmodem);
            if (ALLOK)
            {

                if (romboot_zmodem->header_type == ZDATA)
                {

                    if (zmodem_pos_match(romboot_zmodem))
                    {
                        modem_register_good(romboot_zmodem);
                        romboot_zmodem->moredata = 1;
#ifdef RDEBUG
                        ZM_INFO("got ZDATA for position %ld\n", romboot_zmodem->good_offset);
#endif
                    }
                }
                else if (romboot_zmodem->header_type == ZEOF)
                {
#ifdef RDEBUG
                    ZM_INFO("got ZEOF for position %ld, data_start_offset = 0x%x\n",
                            romboot_zmodem->good_offset, romboot_zmodem->data_start_offset);
#endif
                    if (zmodem_pos_match(romboot_zmodem))
                    {
                        modem_register_good(romboot_zmodem);
                        quit = 1;
                        if ((receive_count_16 % MUSIC_SPI_PAGE_SIZE) != 0)
                        {
                            uint32_t offset_len = 0;
                            if (receive_count_16 % MUSIC_SPI_PAGE_SIZE % 4 != 0)
                                offset_len = 4 - (receive_count_16 % MUSIC_SPI_PAGE_SIZE % 4);
                            music_spi_write_page(((receive_count_16 / MUSIC_SPI_PAGE_SIZE) * MUSIC_SPI_PAGE_SIZE) +
                                                 image_flash_start,
                                                 (uint8_t
                                                  *) (((uint32_t) (romboot_zmodem->buf_pos) / MUSIC_SPI_PAGE_SIZE) *
                                                      MUSIC_SPI_PAGE_SIZE),
                                                 receive_count_16 % MUSIC_SPI_PAGE_SIZE + offset_len);
                        }
                    }
                }
                else
                {
                    if (head_tries < 10)
                    {
                        head_tries++;
                    }
                    else
                    {
                        zmodem_error_set(ZMODEM_TIMEOUT);
                    }
                }
            }

        }

        if (!ALLOK)
        {

            if (zmodem_error_compare(ZMODEM_TIMEOUT))
            {
                /* if TIMEOUT retry but no zmodem_send_ZRPOS send maybe due to buffer read empty */
#ifdef RDEBUG
                ZM_INFO("TIME OUT retry in receive data (%d)\n", timeouts);
#endif
                if (timeouts < 10)
                {
                    romboot_zmodem->moredata = 1;
                    zmodem_error_flush();
                }
                else if (timeouts < 20)
                {
                    romboot_zmodem->moredata = 0;
                    zmodem_error_flush();
                    zmodem_send_ZRPOS(romboot_zmodem);
                }
                else
                {
                    romboot_zmodem->moredata = 0;
                    zmodem_error_flush();
                    quit = 1;
                }
                timeouts++;
            }

            if (zmodem_error_compare(ZMODEM_LONGSP)
                || zmodem_error_compare(ZMODEM_CRCXM) || zmodem_error_compare(ZMODEM_CRC32))
            {

                if (tries < 10)
                {
                    zmodem_error_flush();
                    romboot_zmodem->moredata = 0;
                    zmodem_send_ZRPOS(romboot_zmodem);
                    tries++;
                }
                else
                {
                    zmodem_error_flush();
                    romboot_zmodem->moredata = 0;
                    quit = 1;
                }
            }
        }

    }
    return;
}

static int32_t zmodem_pos_match(ZMCORE * romboot_zmodem)
{
    uint32_t templ = 0;
    int32_t ret = 0;

    templ = romboot_zmodem->header_data[3];
    templ = (templ << 8) | romboot_zmodem->header_data[2];
    templ = (templ << 8) | romboot_zmodem->header_data[1];
    templ = (templ << 8) | romboot_zmodem->header_data[0];

    if (templ == romboot_zmodem->good_offset)
    {
        ret = 1;
    }
    else
    {
#ifdef RDEBUG
        ZM_INFO("failed to match their offset %ld with ours %ld\n", templ, romboot_zmodem->good_offset);
#endif
        ret = 0;
    }
    return (ret);
}

static void zmodem_get_data(ZMCORE * romboot_zmodem)
{
    if (romboot_zmodem->frame_type == ZBIN32)
    {
        zmodem_get_data32(romboot_zmodem);
    }
    else
    {
        zmodem_get_data16(romboot_zmodem);
    }

    if (!ALLOK)
    {
        romboot_zmodem->moredata = 0;
    }
    else
    {
        modem_register_good(romboot_zmodem);
    }

    return;
}


static void zmodem_get_data16(ZMCORE * romboot_zmodem)
{
    int32_t quit = 0;
    CRCXM crc;
    uint32_t their_crc = 0;

    crcxmInit(&crc);

    while (ALLOK && (romboot_zmodem->buf_pos < romboot_zmodem->buf_top) && !quit)
    {
        zmodem_get_next_DLEch(romboot_zmodem);

        if (ALLOK)
        {

            if (romboot_zmodem->got_special)
            {

                if (romboot_zmodem->ch != ZCRCG)
                {
                    romboot_zmodem->moredata = 0;
                }
                crcxmUpdate(&crc, romboot_zmodem->ch);
                zmodem_get_next_DLEch(romboot_zmodem);

                if (ALLOK)
                {
                    their_crc = romboot_zmodem->ch;
                    zmodem_get_next_DLEch(romboot_zmodem);
                }

                if (ALLOK)
                {
                    their_crc = (their_crc << 8) | romboot_zmodem->ch;

                    if (crcxmValue(&crc) != their_crc)
                    {
                        zmodem_error_set(ZMODEM_CRCXM);
                    }
                    else
                    {
                        romboot_zmodem->good_offset = (romboot_zmodem->buf_pos - romboot_zmodem->main_buf);
                        if (begin_receive_file == 1)
                        {
                            romboot_zmodem->good_offset = receive_count_16;
                        }
                        quit = 1;
                    }
                }
            }
            else
            {
                crcxmUpdate(&crc, romboot_zmodem->ch);
                *romboot_zmodem->buf_pos = (uint8_t) romboot_zmodem->ch;
                if (begin_receive_file == 1)
                {
                    if ((receive_count_16 % MUSIC_SPI_PAGE_SIZE) == (MUSIC_SPI_PAGE_SIZE - 1))
                        music_spi_write_page((receive_count_16 + 1 - MUSIC_SPI_PAGE_SIZE) + image_flash_start,
                                             (uint8_t *) ((uint32_t) (romboot_zmodem->buf_pos) - MUSIC_SPI_PAGE_SIZE +
                                                          1), MUSIC_SPI_PAGE_SIZE);
                    receive_count_16++;
                }
                romboot_zmodem->buf_pos++;
                if ((begin_receive_file == 1) && ((receive_count_16 % MUSIC_SPI_SECTOR_SIZE) == 0))
                {
                    romboot_zmodem->buf_pos = (uint8_t *) ZMODEM_RECEIVE_BASEADDR;
                }
            }
        }
    }

    if (romboot_zmodem->buf_pos == romboot_zmodem->buf_top)
    {
        zmodem_error_set(ZMODEM_LONGSP);
    }
    return;
}

static void zmodem_get_data32(ZMCORE * romboot_zmodem)
{
    int32_t quit = 0;
    CRC32 crc;
    uint32_t their_crc = 0;

    //romboot_zmodem->buf_pos = romboot_zmodem->main_buf;
    crc32Init(&crc);

    while (ALLOK && (romboot_zmodem->buf_pos < romboot_zmodem->buf_top) && !quit)
    {
        zmodem_get_next_DLEch(romboot_zmodem);

        if (ALLOK)
        {

            if (romboot_zmodem->got_special)
            {

                if (romboot_zmodem->ch != ZCRCG)
                {
                    romboot_zmodem->moredata = 0;
                }

                crc32Update(&crc, romboot_zmodem->ch);
                zmodem_get_next_DLEch(romboot_zmodem);

                if (ALLOK)
                {
                    their_crc = romboot_zmodem->ch;
                    zmodem_get_next_DLEch(romboot_zmodem);
                }

                if (ALLOK)
                {
                    their_crc = their_crc | ((uint32_t) romboot_zmodem->ch << 8);
                    zmodem_get_next_DLEch(romboot_zmodem);
                }

                if (ALLOK)
                {
                    their_crc = their_crc | ((uint32_t) romboot_zmodem->ch << 16);
                    zmodem_get_next_DLEch(romboot_zmodem);
                }

                if (ALLOK)
                {
                    their_crc = their_crc | ((uint32_t) romboot_zmodem->ch << 24);

                    if (~crc32Value(&crc) != their_crc)
                    {
                        zmodem_error_set(ZMODEM_CRC32);
#ifdef RDEBUG
                        ZM_INFO("got CRC32 error, last good offset %ld\n", romboot_zmodem->good_offset);
#endif
                    }
                    else
                    {
                        romboot_zmodem->good_offset = (romboot_zmodem->buf_pos - romboot_zmodem->main_buf);
                        if (begin_receive_file == 1)
                        {
                            romboot_zmodem->good_offset = receive_count_16;
                        }
                        //romboot_zmodem->main_buf = romboot_zmodem->buf_pos;
                        quit = 1;
                    }
                }
            }
            else
            {
                crc32Update(&crc, romboot_zmodem->ch);
                *romboot_zmodem->buf_pos = (uint8_t) romboot_zmodem->ch;
                if (begin_receive_file == 1)
                {
                    if ((receive_count_16 % MUSIC_SPI_PAGE_SIZE) == (MUSIC_SPI_PAGE_SIZE - 1))
                        music_spi_write_page((receive_count_16 + 1 - MUSIC_SPI_PAGE_SIZE) + image_flash_start,
                                             (uint8_t *) ((uint32_t) (romboot_zmodem->buf_pos) - MUSIC_SPI_PAGE_SIZE +
                                                          1), MUSIC_SPI_PAGE_SIZE);
                    receive_count_16++;
                }
                romboot_zmodem->buf_pos++;
                if ((begin_receive_file == 1) && ((receive_count_16 % MUSIC_SPI_SECTOR_SIZE) == 0))
                {
                    romboot_zmodem->buf_pos = (uint8_t *) ZMODEM_RECEIVE_BASEADDR;
                }
            }
        }
    }

    if (romboot_zmodem->buf_pos == romboot_zmodem->buf_top)
    {
        zmodem_error_set(ZMODEM_LONGSP);
    }
    return;
}


static void zmodem_get_filedata(ZMCORE * romboot_zmodem)
{
    zmodem_get_data(romboot_zmodem);

    if (ALLOK)
    {
        /* add by qxu sparate the file and file info */
        /* this function need improve */
        romboot_zmodem->data_start_offset = romboot_zmodem->buf_pos - romboot_zmodem->main_buf;

        if (FILENAME_MAX < romboot_zmodem->data_start_offset)
        {
            zmodem_error_set(ZMODEM_LONGSP);
            return;
        }

        memcpy((uint8_t *) romboot_zmodem->filename,
               (uint8_t *) romboot_zmodem->main_buf, romboot_zmodem->data_start_offset);

        romboot_zmodem->buf_pos = (uint8_t *) ZMODEM_RECEIVE_BASEADDR;
        romboot_zmodem->filename[FILENAME_MAX - 1] = '\0';

    }

    return;
}


static void zmodem_get_header(ZMCORE * romboot_zmodem)
{
    int32_t count = 0;

    romboot_zmodem->got_header = 0;
    zmodem_get_next_ch(romboot_zmodem);

    while (ALLOK && !romboot_zmodem->got_header)
    {

        while (ALLOK && (romboot_zmodem->ch != ZPAD))
        {
            /* modem_register_bad(romboot_zmodem); */
            zmodem_get_next_ch(romboot_zmodem);
        }

        if (ALLOK)
        {
            count = 1;
            zmodem_get_next_ch(romboot_zmodem);

            while (ALLOK && (romboot_zmodem->ch == ZPAD))
            {
                count++;

                if (count > 2)
                {
                    /* modem_register_bad(romboot_zmodem); */
                }
                zmodem_get_next_ch(romboot_zmodem);
            }

            if (ALLOK && (romboot_zmodem->ch == ZDLE))
            {
                zmodem_get_next_ch(romboot_zmodem);

                if (ALLOK)
                {

                    if (romboot_zmodem->ch == ZBIN)
                    {
                        romboot_zmodem->frame_type = ZBIN;
                        zmodem_get_binary_header(romboot_zmodem);
                    }
                    else if (romboot_zmodem->ch == ZBIN32)
                    {
                        romboot_zmodem->frame_type = ZBIN32;
                        zmodem_get_bin32_header(romboot_zmodem);
                    }
                    else if ((romboot_zmodem->ch == ZHEX) && (count >= 2))
                    {
                        romboot_zmodem->frame_type = ZHEX;
                        zmodem_get_hex_header(romboot_zmodem);
                    }
                }
            }
        }
    }

    if (romboot_zmodem->got_header)
    {
#ifdef RDEBUG
        ZM_INFO("received header_type %x\n", romboot_zmodem->header_type);
#endif
    }
    else
    {
#ifdef RDEBUG
        ZM_INFO("received header_type %x timeout?\n", romboot_zmodem->header_type);
#endif
        if (romboot_zmodem->header_type == 6)
        {
            zmodem_error_set(ZMODEM_TIMEOUT);
        }

    }
    return;
}

static void zmodem_get_hex_header(ZMCORE * romboot_zmodem)
{
    CRCXM crc;
    uint32_t their_crc = 0;

#ifdef RDEBUG
    ZM_INFO("zmodem_get_hex_header\n");
#endif

    zmodem_get_next_hexch(romboot_zmodem);

    while (ALLOK && !romboot_zmodem->got_header)
    {
        crcxmInit(&crc);
        romboot_zmodem->header_type = (uint8_t) romboot_zmodem->ch;
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_get_next_hexch(romboot_zmodem);

        if (ALLOK)
        {
            romboot_zmodem->header_data[0] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_hexch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[1] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_hexch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[2] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_hexch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[3] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_hexch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = romboot_zmodem->ch;
            zmodem_get_next_hexch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = (their_crc << 8) | romboot_zmodem->ch;

            if (crcxmValue(&crc) != their_crc)
            {
                zmodem_error_set(ZMODEM_CRCXM);
            }
            else
            {
                /*
                   check for CR (optional) & LF
                 */
                zmodem_get_next_ch(romboot_zmodem);

                if (ALLOK && (romboot_zmodem->ch == 0x0d))
                {
                    zmodem_get_next_ch(romboot_zmodem);
                }
                if (ALLOK)
                {
                    romboot_zmodem->got_header = 1;
                }
            }
        }
    }
    return;
}

static void zmodem_get_binary_header(ZMCORE * romboot_zmodem)
{
    CRCXM crc;
    uint32_t their_crc = 0;

#ifdef RDEBUG
    ZM_INFO("zmodem_get_hex_header\n");
#endif
    zmodem_get_next_DLEch(romboot_zmodem);

    while (ALLOK && !romboot_zmodem->got_header)
    {
        crcxmInit(&crc);
        romboot_zmodem->header_type = (uint8_t) romboot_zmodem->ch;
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_get_next_DLEch(romboot_zmodem);

        if (ALLOK)
        {
            romboot_zmodem->header_data[0] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[1] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[2] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[3] = (uint8_t) romboot_zmodem->ch;
            crcxmUpdate(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = romboot_zmodem->ch;
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = (their_crc << 8) | romboot_zmodem->ch;

            if (crcxmValue(&crc) != their_crc)
            {
                zmodem_error_set(ZMODEM_CRCXM);
            }
            else
            {
                romboot_zmodem->got_header = 1;
            }
        }

    }
    return;
}

static void zmodem_get_bin32_header(ZMCORE * romboot_zmodem)
{
    CRC32 crc;
    uint32_t their_crc = 0;

#ifdef RDEBUG
    ZM_INFO("zmodem_get_hex_header\n");
#endif

    zmodem_get_next_DLEch(romboot_zmodem);

    while (ALLOK && !romboot_zmodem->got_header)
    {
        crc32Init(&crc);
        romboot_zmodem->header_type = (uint8_t) romboot_zmodem->ch;
        crc32Update(&crc, romboot_zmodem->ch);
        zmodem_get_next_DLEch(romboot_zmodem);

        if (ALLOK)
        {
            romboot_zmodem->header_data[0] = (uint8_t) romboot_zmodem->ch;
            crc32Update(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[1] = (uint8_t) romboot_zmodem->ch;
            crc32Update(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[2] = (uint8_t) romboot_zmodem->ch;
            crc32Update(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            romboot_zmodem->header_data[3] = (uint8_t) romboot_zmodem->ch;
            crc32Update(&crc, romboot_zmodem->ch);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = (uint32_t) romboot_zmodem->ch;
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = their_crc | ((uint32_t) romboot_zmodem->ch << 8);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = their_crc | ((uint32_t) romboot_zmodem->ch << 16);
            zmodem_get_next_DLEch(romboot_zmodem);
        }

        if (ALLOK)
        {
            their_crc = their_crc | ((uint32_t) romboot_zmodem->ch << 24);
            if (~crc32Value(&crc) != their_crc)
            {
                zmodem_error_set(ZMODEM_CRC32);
            }
            else
            {
                romboot_zmodem->got_header = 1;
            }
        }
    }
    return;
}

static void zmodem_get_next_hexch(ZMCORE * romboot_zmodem)
{
    int32_t temp_ch = 0;

    zmodem_get_next_ch(romboot_zmodem);

    if (ALLOK)
    {

        if ((romboot_zmodem->ch <= 0x39) && (romboot_zmodem->ch >= 0x30))
        {
            temp_ch = (romboot_zmodem->ch - 0x30);
        }
        else if ((romboot_zmodem->ch >= 0x61) && (romboot_zmodem->ch <= 0x66))
        {
            temp_ch = (romboot_zmodem->ch - 0x61) + 0x0a;
        }
        else
        {
            zmodem_error_set(ZMODEM_BADHEX);
        }

        if (ALLOK)
        {
            zmodem_get_next_ch(romboot_zmodem);
        }

        if (ALLOK)
        {
            temp_ch = temp_ch << 4;

            if ((romboot_zmodem->ch <= 0x39) && (romboot_zmodem->ch >= 0x30))
            {
                romboot_zmodem->ch = (romboot_zmodem->ch - 0x30);
            }
            else if ((romboot_zmodem->ch >= 0x61) && (romboot_zmodem->ch <= 0x66))
            {
                romboot_zmodem->ch = (romboot_zmodem->ch - 0x61) + 0x0a;
            }
            else
            {
                zmodem_error_set(ZMODEM_BADHEX);
            }
        }

        if (ALLOK)
        {
            romboot_zmodem->ch = romboot_zmodem->ch | temp_ch;
        }
    }
    return;
}

static void zmodem_get_next_DLEch(ZMCORE * romboot_zmodem)
{
    romboot_zmodem->got_special = 0;
    zmodem_get_next_ch(romboot_zmodem);

    if (ALLOK)
    {

        if (romboot_zmodem->ch == ZDLE)
        {
            zmodem_get_next_ch(romboot_zmodem);

            if (ALLOK)
            {
                romboot_zmodem->got_special = 0;

                if (romboot_zmodem->ch == ZRUB0)
                {
                    romboot_zmodem->ch = 0x7f;
                }
                else if (romboot_zmodem->ch == ZRUB1)
                {
                    romboot_zmodem->ch = 0xff;
                }
                else if (((romboot_zmodem->ch & 0x40) != 0) && ((romboot_zmodem->ch & 0x20) == 0))
                {
                    romboot_zmodem->ch &= 0xbf;
                }
                else
                {
                    romboot_zmodem->got_special = 1;
                }
            }
        }
    }
    return;
}

static void zmodem_get_next_ch(ZMCORE * romboot_zmodem)
{
    uint8_t buf;

    extmodem_getblock(romboot_zmodem, &buf);

    if (ALLOK)
    {
        romboot_zmodem->ch = buf;
    }

    return;
}

static void zmodem_send_header(ZMCORE * romboot_zmodem)
{
#ifdef RDEBUG
    ZM_INFO("sending header %x\n", romboot_zmodem->header_type);
#endif
    if (romboot_zmodem->frame_type == ZHEX)
    {
        zmodem_send_hex_header(romboot_zmodem);
    }
    else if (romboot_zmodem->frame_type == ZBIN)
    {
        zmodem_send_bin_header(romboot_zmodem);
    }
    else if (romboot_zmodem->frame_type == ZBIN32)
    {
        zmodem_send_bin32_header(romboot_zmodem);
    }

    return;
}

static void zmodem_send_hex_header(ZMCORE * romboot_zmodem)
{
    CRCXM crc;

    crcxmInit(&crc);

    /* send ZPAD, ZPAD, ZDLE ... */
    romboot_zmodem->ch = ZPAD;
    zmodem_send_char(romboot_zmodem);

    if (ALLOK)
    {
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = ZDLE;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->frame_type;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_type;
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[0];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[1];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[2];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[3];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = crcxmHighbyte(&crc);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = crcxmLowbyte(&crc);
        zmodem_send_hex_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = 0x0d;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = 0x0a;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        if ((romboot_zmodem->header_type != ZFIN) && (romboot_zmodem->header_type != ZACK))
        {
            romboot_zmodem->ch = 0x11;
            zmodem_send_char(romboot_zmodem);
        }
    }
    return;
}

static void zmodem_send_bin_header(ZMCORE * romboot_zmodem)
{
    CRCXM crc;

    crcxmInit(&crc);
    romboot_zmodem->ch = ZPAD;
    zmodem_send_char(romboot_zmodem);

    if (ALLOK)
    {
        romboot_zmodem->ch = ZDLE;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->frame_type;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_type;
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[0];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[1];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[2];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[3];
        crcxmUpdate(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = crcxmHighbyte(&crc);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = crcxmLowbyte(&crc);
        zmodem_send_DLE_char(romboot_zmodem);
    }
    return;
}

static void zmodem_send_bin32_header(ZMCORE * romboot_zmodem)
{
    CRC32 crc;

    crc32Init(&crc);
    romboot_zmodem->ch = ZPAD;
    zmodem_send_char(romboot_zmodem);

    if (ALLOK)
    {
        romboot_zmodem->ch = ZDLE;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->frame_type;
        zmodem_send_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_type;
        crc32Update(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[0];
        crc32Update(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[1];
        crc32Update(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[2];
        crc32Update(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = romboot_zmodem->header_data[3];
        crc32Update(&crc, romboot_zmodem->ch);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = ~(uint8_t) crc32Byte4(&crc);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = ~(uint8_t) crc32Byte3(&crc);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = ~(uint8_t) crc32Byte2(&crc);
        zmodem_send_DLE_char(romboot_zmodem);
    }

    if (ALLOK)
    {
        romboot_zmodem->ch = ~(uint8_t) crc32Byte1(&crc);
        zmodem_send_DLE_char(romboot_zmodem);
    }
    return;
}

static void zmodem_send_char(ZMCORE * romboot_zmodem)
{
    uint8_t buf = 0;

    buf = (uint8_t) romboot_zmodem->ch;

    extmodem_writeblock(romboot_zmodem, &buf, 1);
    return;
}

static void zmodem_send_DLE_char(ZMCORE * romboot_zmodem)
{
    int32_t temp_ch = 0;

    if (needsDLE(romboot_zmodem->ch))
    {
        temp_ch = romboot_zmodem->ch;
        romboot_zmodem->ch = ZDLE;
        zmodem_send_char(romboot_zmodem);
        if (ALLOK)
        {
            if (temp_ch == 0x7f)
            {
                romboot_zmodem->ch = ZRUB0;
            }
            else if (temp_ch == 0xff)
            {
                romboot_zmodem->ch = ZRUB1;
            }
            else
            {
                romboot_zmodem->ch = temp_ch | 0x40;
            }
            zmodem_send_char(romboot_zmodem);
        }
    }
    else
    {
        zmodem_send_char(romboot_zmodem);
    }
    return;
}

static void zmodem_send_hex_char(ZMCORE * romboot_zmodem)
{
    uint8_t temp_ch = 0;

    temp_ch = romboot_zmodem->ch;
#if 0
    romboot_zmodem->ch = hexdigit[(temp_ch >> 4) & 0x0f];
#endif
    if ((temp_ch >> 4) < 10)
    {
        romboot_zmodem->ch = (temp_ch >> 4) + 0x30;
    }
    else
    {
        romboot_zmodem->ch = (temp_ch >> 4) + 0x61 - 10;
    }

    zmodem_send_char(romboot_zmodem);
    if (ALLOK)
    {
#if 0
        romboot_zmodem->ch = hexdigit[temp_ch & 0x0f];
#endif
        if ((temp_ch & 0xf) < 10)
        {
            romboot_zmodem->ch = (temp_ch & 0xf) + 0x30;
        }
        else
        {
            romboot_zmodem->ch = (temp_ch & 0xf) + 0x61 - 10;
        }
        zmodem_send_char(romboot_zmodem);
    }
    return;
}

static void zmodem_send_ZRINIT(ZMCORE * romboot_zmodem)
{
    romboot_zmodem->frame_type = ZHEX;
    romboot_zmodem->header_type = ZRINIT;
    romboot_zmodem->header_data[0] = 0x00;
    romboot_zmodem->header_data[1] = 0x00;
    romboot_zmodem->header_data[2] = 0x00;
    romboot_zmodem->header_data[3] = CANOVIO | CANFDX | CANFC32;
    zmodem_send_header(romboot_zmodem);
    return;
}

static void zmodem_send_ZKIP(ZMCORE * romboot_zmodem)
{
    romboot_zmodem->frame_type = ZHEX;
    romboot_zmodem->header_type = ZSKIP;
    romboot_zmodem->header_data[0] = 0x00;
    romboot_zmodem->header_data[1] = 0x00;
    romboot_zmodem->header_data[2] = 0x00;
    romboot_zmodem->header_data[3] = 0x00;
    zmodem_send_header(romboot_zmodem);
    return;
}


static void zmodem_send_ZRPOS(ZMCORE * romboot_zmodem)
{
    long templ = 0;

    romboot_zmodem->frame_type = ZHEX;
    romboot_zmodem->header_type = ZRPOS;
    templ = romboot_zmodem->good_offset;
    romboot_zmodem->header_data[0] = (uint8_t) (templ & 0xff);
    templ = templ >> 8;
    romboot_zmodem->header_data[1] = (uint8_t) (templ & 0xff);
    templ = templ >> 8;
    romboot_zmodem->header_data[2] = (uint8_t) (templ & 0xff);
    templ = templ >> 8;
    romboot_zmodem->header_data[3] = (uint8_t) (templ & 0xff);
    //extModemClearInbound(romboot_zmodem);
    zmodem_send_header(romboot_zmodem);
    return;
}

static void zmodem_send_ZFIN(ZMCORE * romboot_zmodem)
{
    romboot_zmodem->frame_type = ZHEX;
    romboot_zmodem->header_type = ZFIN;
    romboot_zmodem->header_data[0] = 0x00;
    romboot_zmodem->header_data[1] = 0x00;
    romboot_zmodem->header_data[2] = 0x00;
    romboot_zmodem->header_data[3] = 0x00;
    zmodem_send_header(romboot_zmodem);
    return;
}


static void zmodem_get_OO(ZMCORE * romboot_zmodem)
{
    zmodem_get_next_ch(romboot_zmodem);

    if (ALLOK)
    {
        zmodem_get_next_ch(romboot_zmodem);
    }

    return;
}

void zmodem_main()
{
    ZMCORE romboot_zmodem;
    uint32_t zmodem_delay = 0;
    int i;


    //serial_init();
    printf("erase flash start:0x%x size:0x%x\n", image_flash_start, image_flash_size);
    music_spi_init();
    for (i = 0; i < (image_flash_size + MUSIC_SPI_SECTOR_SIZE - 1) / MUSIC_SPI_SECTOR_SIZE; i++)
        music_spi_sector_erase(i * MUSIC_SPI_SECTOR_SIZE + image_flash_start);

    memset(&romboot_zmodem, 0, sizeof(romboot_zmodem));
    //printf("Entering ZMODEM download:\n");

    /* call zmodem function */
    romboot_zmodem_init(&romboot_zmodem);

    for (zmodem_delay = 0;; zmodem_delay++)
    {
        if (!(zmodem_delay % 100))
        {
            printf("Waiting for file transfer via ZMODEM...\n");
            begin_receive_file = 0;
            zmodem_error_flush();
            romboot_zmodem_receive(&romboot_zmodem);
        }
        //udelay(10000);
    }
}

void extmodem_getblock(ZMCORE * zmcore, uint8_t * buf)
{
    uint8_t ch = 0;
    ch = serial_getc();
    *buf = ch;

    return;
}


void extmodem_writeblock(ZMCORE * zmcore, uint8_t * buf, int32_t max)
{
    int32_t loop = 0;
    for (loop = 0; loop < max; loop++)
    {
        serial_putc(*(buf + loop));
    }
    return;
}
