/*
 * Copyright (c) 2012 Qualcomm Atheros, Inc..
 * All Rights Reserved.
 * Qualcomm Atheros Confidential and Proprietary.
 */

#include "gmac.h"

ath_header_t vivo_aligned_header;
uint16_t cpu_tpid_value = 0x9200;
#define virtual_to_physical(_x) ((uint32_t)(_x) - 0xc0000000)


static gmac_tpd_t *tpd_desc;
static gmac_rfd_t *rfd_desc;
static gmac_rrd_t *rrd_desc;

static uint8_t rx_id, rx_next_id;
static uint8_t _desc_buf[DESC_BUF_SIZE] = { 0 };
static uint8_t _rx_buf[VIVO_NUM_RX_DESC][VIVO_RX_BUF_SIZE + DMA_ALIGN_SIZE];
static uint8_t _tx_buf[VIVO_NUM_RX_DESC][VIVO_RX_BUF_SIZE + DMA_ALIGN_SIZE];

static uint8_t *desc_buf;
static uint8_t *rx_buf;


static void gmac_desc_setup(void)
{
    uint32_t i;

    desc_buf = KSEG1ADDR((uint8_t *) (virtual_to_physical(_desc_buf)));
    rx_buf = KSEG1ADDR((uint8_t *) (virtual_to_physical(_rx_buf)));

    memset(rx_buf, 0, sizeof(VIVO_NUM_RX_DESC * (VIVO_RX_BUF_SIZE + DMA_ALIGN_SIZE)));
    memset(desc_buf, 0, sizeof(DESC_BUF_SIZE));

    /* TPD */
    tpd_desc = (gmac_tpd_t *) DMA_ALIGN_ADDR((uint32_t) desc_buf);
    gmac_reg_wr(REGISTER_TX_BASE_ADDR_HI, 0);
    gmac_reg_wr(REGISTER_NTPD_HEAD_ADDR_LO, PHYSADDR(tpd_desc));
    gmac_reg_wr(REGISTER_TPD_RING_SIZE, (VIVO_NUM_TX_DESC & TPD_RING_SIZE_MASK));

    /* RFD */
    rfd_desc = (gmac_rfd_t *) DMA_ALIGN_ADDR((uint32_t) tpd_desc + sizeof(gmac_tpd_t) * VIVO_NUM_TX_DESC);

    for (i = 0; i < VIVO_NUM_RX_DESC; i++)
    {
        rfd_desc[i].buffer_address = cpu_to_le32(PHYSADDR(DMA_ALIGN_ADDR(rx_buf +
                                                                         i * (VIVO_RX_BUF_SIZE + DMA_ALIGN_SIZE))));
        rfd_desc[i].buffer_address_high = 0;
    }

    gmac_reg_wr(REGISTER_RX_BASE_ADDR_HI, 0);
    gmac_reg_wr(REGISTER_RFD0_HEAD_ADDR_LO, PHYSADDR(rfd_desc));
    gmac_reg_wr(REGISTER_RFD_RING_SIZE, (VIVO_NUM_RX_DESC & RFD_RING_SIZE_MASK));
    gmac_reg_wr(REGISTER_RX_BUF_SIZE, (VIVO_RX_MTU + 1000) & RX_BUF_SIZE_MASK);

    /* RRD */
    rrd_desc = (gmac_rrd_t *) DMA_ALIGN_ADDR((uint32_t) rfd_desc + sizeof(gmac_rfd_t) * VIVO_NUM_RX_DESC);

    gmac_reg_wr(REGISTER_RRD0_HEAD_ADDR_LO, PHYSADDR(rrd_desc));
    gmac_reg_wr(REGISTER_RRD_RING_SIZE, (VIVO_NUM_RX_DESC & RRD_RING_SIZE_MASK));

    /* Load all of base address above */
    gmac_reg_wr(REGISTER_LOAD_PTR, 1);

    rx_id = 0;
    rx_next_id = 1;
    gmac_reg_wr(REGISTER_MAILBOX_RFD0_PROD_IDX, (rx_next_id & MAILBOX_RFDX_PROD_IDX_MASK));

    return;
}

static void gmac_tx_setup(void)
{
    uint32_t data = 0;

    gmac_reg_wr(REGISTER_TX_TSO_OFFLOAD_THRESH, 1500);
    data = (VIVO_TPD_BURST & TXQ_NUM_TPD_BURST_MASK) << TXQ_NUM_TPD_BURST_SHIFT;
    data |= TXQ_CONTROL_ENH_MODE;
    data |= (128 * (2 ^ VIVO_DMAR_BLOCK) & TXQ_TXF_BURST_NUM_MASK) << TXQ_TXF_BURST_NUM_SHIFT;
    gmac_reg_wr(REGISTER_TXQ_CTRL, data);

    return;
}

static void gmac_rx_setup(void)
{
    uint32_t data = 0;

    data = (VIVO_RFD_BURST & RXQ_RFD_BURST_NUM_MASK) << RXQ_RFD_BURST_NUM_SHIFT;
    gmac_reg_wr(REGISTER_RXQ_CTRL, data);

    return;
}

static void gmac_dma_setup(void)
{
    uint32_t data = 0;

    data |= DMA_CONTROL_DMAR_OUT_ORDER;
    data |= (((uint32_t) VIVO_DMAR_BLOCK) & DMA_CONTROL_DMAR_BURST_LEN_MASK) << DMA_CONTROL_DMAR_BURST_LEN_SHIFT;
    data |= (((uint32_t) VIVO_DMAW_BLOCK) & DMA_CONTROL_DMAW_BURST_LEN_MASK) << DMA_CONTROL_DMAW_BURST_LEN_SHIFT;
    data |= (((uint32_t) VIVO_DMAR_DLY_CNT) & DMA_CONTROL_DMAR_DLY_CNT_MASK) << DMA_CONTROL_DMAR_DLY_CNT_SHIFT;
    data |= (((uint32_t) VIVO_DMAW_DLY_CNT) & DMA_CONTROL_DMAW_DLY_CNT_MASK) << DMA_CONTROL_DMAW_DLY_CNT_SHIFT;

    gmac_reg_wr(REGISTER_DMA_CTRL, data);

    return;
}

static void gmac_mac_setup(void)
{
    uint32_t data = 0;

    /*mac reset */
    data = gmac_reg_rd(REGISTER_MASTER_CTRL);
    gmac_reg_wr(REGISTER_MASTER_CTRL, (data | MASTER_CONTROL_SOFT_RST));

    udelay(200 * 1000);
    //wmb();

    /*vivoeros header disable */
    data = gmac_reg_rd(REGISTER_ATHEROS_HEADER_CTRL);
    data &= ~ATHEROS_HEADER_EN;
    data &= ~ATHEROS_HEADER_CNT_EN;
    gmac_reg_wr(REGISTER_ATHEROS_HEADER_CTRL, data);

    /*big endian enable */
    data = gmac_reg_rd(REGISTER_AXI_MAST_CTRL);
    gmac_reg_wr(REGISTER_AXI_MAST_CTRL, (data | AXI_MAST_ENDIAN_SWAP_EN));

    /*rfd/tpd/rrd setup */
    gmac_desc_setup();

    /* MTU setup */
    gmac_reg_wr(REGISTER_MTU, VIVO_RX_MTU);

    gmac_tx_setup();
    gmac_rx_setup();
    gmac_dma_setup();

    /*mac setup */
    data = 0;
    data |= (MAC_CONTROL_TX_EN | MAC_CONTROL_RX_EN);
    data |= (MAC_CONTROL_TX_FLOW | MAC_CONTROL_RX_FLOW);
    data |= ((VIVO_MAC_SPEED_1000 & MAC_CONTROL_SPEED_MASK) << MAC_CONTROL_SPEED_SHIFT);
    data |= MAC_CONTROL_DUPLX;
    data |= (MAC_CONTROL_ADD_CRC | MAC_CONTROL_PAD);
    data |= ((VIVO_PREAMBLE_LEN & MAC_CONTROL_PRMLEN_MASK) << MAC_CONTROL_PRMLEN_SHIFT);
    data |= MAC_CONTROL_BC_EN;
    data |= MAC_CONTROL_PROMIS_EN;
    data |= MAC_CONTROL_RX_CHKSUM_EN;
    data |= MAC_CONTROL_SINGLE_PAUSE_EN;
    gmac_reg_wr(REGISTER_MAC_CTRL, data);

    /*txq enable */
    data = gmac_reg_rd(REGISTER_TXQ_CTRL);
    gmac_reg_wr(REGISTER_TXQ_CTRL, (data | TXQ_CONTROL_EN));

    /*rxq enable */
    data = gmac_reg_rd(REGISTER_RXQ_CTRL);
    gmac_reg_wr(REGISTER_RXQ_CTRL, (data | RXQ_CONTROL_EN));

}

static void gmac_pkt_tx(gmac_tx_bd_t * tb)
{
    uint32_t i = 0;
    uint32_t data = 0, hw_next_clean = 0;
    gmac_tpd_t *this_tpd = &tpd_desc[tb->tx_id];

    memset(this_tpd, 0, sizeof(gmac_tpd_t));

    this_tpd->buffer_address = cpu_to_le32(PHYSADDR(tb->buf));
    this_tpd->data0 = cpu_to_le32(tb->len & 0xffff);
    this_tpd->data1 = cpu_to_le32(1 << TPD_EOP_SHIFT);

    data = (tb->next_id & MAILBOX_PRIO_PROD_IDX_MASK) << MAILBOX_NTPD_PROD_IDX_SHIFT;
    gmac_reg_wr(REGISTER_MAILBOX_PRIO_PROD_IDX, data);

    for (i = 0; i < 30000; i++)
    {
        data = gmac_reg_rd(REGISTER_MAILBOX_PRIO_CONS_IDX);
        hw_next_clean = (data >> MAILBOX_NTPD_CONS_IDX_SHIFT) & MAILBOX_PRIO_CONS_IDX_MASK;

        if (hw_next_clean != tb->tx_id)
        {
            break;
        }

        udelay(100);
    }

    if (i == 30000)
    {
        printf("###gmac_pkt_tx timeout###\n\r");
    }
}

static uint8_t gmac_is_rx_pkt(uint32_t * buf_addr, uint32_t * pkt_size)
{
    uint32_t rrs_data3 = le32_to_cpu(rrd_desc[rx_id].data3);

    if (RRS_RXD_IS_VALID(rrs_data3))
    {
        *pkt_size = (rrs_data3 >> RRS_PKT_SIZE_SHIFT) & RRS_PKT_SIZE_MASK;
        *buf_addr = DMA_ALIGN_ADDR((rx_buf + rx_id * (VIVO_RX_BUF_SIZE + DMA_ALIGN_SIZE)));
        return 1;
    }

    return 0;
}

static void gmac_rx_clean(void)
{
    rrd_desc[rx_id].data3 &= ~(cpu_to_le32(1 << RRS_RXD_UPDATED_SHIFT));
    rx_id = (rx_id + 1) % VIVO_NUM_RX_DESC;
    rx_next_id = (rx_next_id + 1) % VIVO_NUM_RX_DESC;

    gmac_reg_wr(REGISTER_MAILBOX_RFD0_PROD_IDX, (rx_next_id & MAILBOX_RFDX_PROD_IDX_MASK));
}

void gmac_init(void)
{
    //printf("gmac_init \n\r");
#ifdef MUSIC_FPGA_BOARD
    switch_port_init();
#else
    switch_reg_wr(0x100, 0);
#endif
    gmac_mac_setup();

#ifndef MUSIC_FPGA_BOARD
    switch_reg_wr(0x100, 0x7e);
#endif
    udelay(300 * 1000);

}


void gmac_exit(void)
{
}

void set_cpu_tpid(uint16_t tpid_value)
{
    cpu_tpid_value = tpid_value;
}

uint32_t rx_pkt_has_ath_header(uint8_t * pkt_buf)
{
    if ((*(pkt_buf + DA_SA_MAC_LEN) == ((cpu_tpid_value & 0xff00) >> 8)) &&
        (*(pkt_buf + DA_SA_MAC_LEN + 1) == (cpu_tpid_value & 0xff)))
        return 1;
    else
        return 0;
}

uint32_t tx_pkt_has_ath_header()
{
    if ((*ath_header == ((cpu_tpid_value & 0xff00) >> 8)) && (*(ath_header + 1) == (cpu_tpid_value & 0xff)))
        return 1;
    else
        return 0;
}

void set_ath_header_to_zero()
{
    *ath_header = 0x0;
    *(ath_header + 1) = 0x0;
    return;
}

void gmac_send(void *pkt, int length)
{
    static uint32_t tx_id = 0;
    gmac_tx_bd_t tb;

#if 1
    uint8_t *tx_buf = KSEG1ADDR((uint8_t *) (virtual_to_physical(_tx_buf[tx_id])));
#if 0
    int i;
    printf("\nsend %d bytes packet!\n\r", length);
    for (i = 0; i < 64; i++)
    {
        if (i % 8 == 0)
            printf("\n\r");
        printf("%2.2x ", *((uint8_t *) pkt + i));
    }
    printf("\n\r");
#endif
    if (tx_pkt_has_ath_header())
    {
        memcpy(tx_buf, pkt, DA_SA_MAC_LEN);
        memcpy(tx_buf + DA_SA_MAC_LEN, ath_header, DNA_HEADER_LEN);
        memcpy(tx_buf + DA_SA_MAC_LEN + DNA_HEADER_LEN, pkt + DA_SA_MAC_LEN, length - DA_SA_MAC_LEN);
        tb.len = length + DNA_HEADER_LEN;
    }
    else
    {
        memcpy(tx_buf, pkt, length);
        tb.len = length;
    }
    tb.buf = tx_buf;
#else
    flush_cache((uint32_t) uip_buf, uip_len);
    tb.buf = uip_buf;
#endif

    //tb.len = uip_len;
    tb.tx_id = tx_id;
    tx_id = (tx_id + 1) % VIVO_NUM_TX_DESC;
    tb.next_id = tx_id;

    gmac_pkt_tx(&tb);
}


unsigned int gmac_poll(uint8_t * buf, uint32_t buf_size)
{
    uint32_t pkt_size = 0, buf_addr;

    if (gmac_is_rx_pkt(&buf_addr, &pkt_size))
    {
        if (0)
        {
            uint8_t *pkt_p = (uint8_t *) buf_addr;
            int i;
            printf("\nreceived %d bytes packet!\n\r", pkt_size);
            for (i = 0; i < 64; i++)
            {
                if (i % 8 == 0)
                    printf("\n\r");
                printf("%2.2x ", *(pkt_p + i));
            }
            printf("\n\r");

        }
        //extract 4bytes crc length
        pkt_size -= VIVO_PKT_CRC_SIZE;
        if (rx_pkt_has_ath_header((uint8_t *) buf_addr))
        {
            if (pkt_size > (buf_size + DNA_HEADER_LEN))
            {
                printf("gmac_poll(void) pktsize %d with ath-header\n\r", pkt_size);
                gmac_rx_clean();
                return 0;
            }
            else if (pkt_size < DNA_HEADER_LEN)
            {
                printf("rx %d B is less than DNA_HEADER_LEN!\n\r", pkt_size);
                gmac_rx_clean();
                return 0;
            }

            memcpy(buf, (void *)buf_addr, DA_SA_MAC_LEN);
            memcpy(buf + DA_SA_MAC_LEN, (void *)(buf_addr + DA_SA_MAC_LEN + DNA_HEADER_LEN),
                   pkt_size - DA_SA_MAC_LEN - DNA_HEADER_LEN);
            memcpy(ath_header, (void *)(buf_addr + DA_SA_MAC_LEN), DNA_HEADER_LEN);
            pkt_size -= DNA_HEADER_LEN;
        }
        else
        {
            if (pkt_size > buf_size)
            {
                printf("gmac_poll(void) pktsize %d without ath-header dst size %d\n\r", pkt_size, buf_size);
                gmac_rx_clean();
                return 0;
            }
            memcpy(buf, (void *)buf_addr, pkt_size);

        }
        gmac_rx_clean();
        return pkt_size;
    }
    return 0;
}
