#include <stdio.h>
#include <assert.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

#define FILE_NAME_LEN       128
#define MAXFILE_LENGTH      0x100000

void show_usage()
{
    printf("usage:\n");
    printf("    image_pad [input_file] [output_file] [output_lenth]\n");
    printf("    [input_file]: the input file name\n");
    printf("    [output_file]: the output file name\n");
    printf("    [output_lenth]: total output file lenght in bytes\n");
}


int main(int argc, char *argv[])
{
    int i;
    struct stat buf;
    int in_file_fd, out_file_fd;
    char checksum, temp_value, *temp_p, *file_buffer_p = NULL;
    unsigned file_length = 0, out_length = 0;
    struct tm *t_tm;
    int current_argv_offset = 1;
    char input_file[FILE_NAME_LEN], output_file[FILE_NAME_LEN];


    if (argc != 4)
    {
        show_usage();
        return -1;
    }


    if (strlen(argv[1]) >= FILE_NAME_LEN)
    {
        printf("input_file:[%s] length exceed FILE_NAME_LEN:%d\n", argv[1], FILE_NAME_LEN);
        return -1;
    }
    strcpy(input_file, argv[1]);


    if (strlen(argv[2]) >= FILE_NAME_LEN)
    {
        printf("input_file:[%s] length exceed FILE_NAME_LEN:%d\n", argv[2], FILE_NAME_LEN);
        return -1;
    }
    strcpy(output_file, argv[2]);


    out_length = atoi(argv[3]);


    printf("inputfile name:%s\n", input_file);
    printf("outputfile name:%s\n", output_file);

    if (stat(input_file, &buf) < 0)
    {
        printf("Error: can not access input file [%s]!\n", input_file);
        return -1;
    }

    file_length = (unsigned int)buf.st_size;

    printf("file length of %s :%d bytes\n", input_file, (int)buf.st_size);


    if (file_length > out_length)
    {
        printf("paddfile length %d bytes is less than input file lenght\n", out_length);
        return -1;
    }

    if (out_length > MAXFILE_LENGTH)
    {
        printf("paddfile length %d is too large, should less than 0x%x bytes\n", out_length, MAXFILE_LENGTH);
        return -1;
    }
    printf("outputfile length:%d bytes\n", out_length);


    in_file_fd = open(input_file, O_RDONLY);
    if (in_file_fd == -1)
    {
        printf("open %s error\n", input_file);
        return -1;
    }

    file_buffer_p = malloc(out_length);
    if (file_buffer_p == NULL)
    {
        printf("malloc %d bytes failed for file_buffer_p!\n", out_length);
        return;
    }
    memset(file_buffer_p, 0xff, out_length);


    if (read(in_file_fd, file_buffer_p, file_length) != file_length)
    {
        printf("inpute file read failed\n");
        close(in_file_fd);
        free(file_buffer_p);
        return -1;
    }

    close(in_file_fd);


    out_file_fd = open(output_file, O_RDWR | O_CREAT);
    if (out_file_fd == -1)
    {
        printf("open %s error\n", output_file);
        return -1;
    }

    if (write(out_file_fd, (char *)file_buffer_p, out_length) != out_length)
    {
        printf("write output file error\n");
        close(out_file_fd);
        return -1;
    }
    close(out_file_fd);

    return 0;
}
