/* 
 * Copyright (c) 2012 Qualcomm Atheros, Inc.. 
 * All Rights Reserved. 
 * Qualcomm Atheros Confidential and Proprietary. 
 */

#include "music_spi.h"
#include "printf_api.h"

/*
 * executable at 0xa0001100
 * + 1M for the boot code (+1M for the boot code copy)
 * Debug vars at 0xa0205000
 * stack at 0xa1000000
 */
//#define ARS124_AHB_CLOCK 25000000
//#define ARS124_SPI_READDATA_CLOCK 12500000

#define ARS124_AHB_CLOCK 300000000/2
#define ARS124_SPI_READDATA_CLOCK 40000000

void (*spi_init_function_p) (void);
//void (*spi_sector_erase_function_p)(uint32_t addr);
void (*spi_sector_erase_function_p) (void);
//void (*spi_write_page_function_p)(uint32_t addr, uint8_t *data, int len);
void (*spi_write_page_function_p) (void);
void (*spi_common_function_p) (void);

extern uint32_t tiks_per_mseconds;

static uint32_t flash_write_addr;
static uint8_t *flash_write_data;
static int flash_write_len;

const board_spi_flash_info_t spi_flash_array[] = {
    {SPI_MX25L25635E_ID, 0x2000000},
    {SPI_MX25L6406E_ID, 0x800000},
    {SPI_M25Q128_ID, 0x1000000},
    {SPI_M25P64_ID, 0x800000},
    {SPI_M25P128_ID, 0x1000000},
    {SPI_MX25L128_ID, 0x1000000}
};

#define SPI_FLASH_ARRAY_CNT (sizeof(spi_flash_array)/sizeof(board_spi_flash_info_t))
#define SIZE_1K 0x400

uint32_t global_flash_size;
uint32_t global_image_a_offset;
uint32_t global_image_b_offset;
uint32_t global_running_offset;
uint32_t global_ctrl_block_offset;
uint32_t global_boot_cfg_offset;
uint32_t global_user_cfg_offset;

const board_partition_info_t partition_info[] = {
/* flash_size, loader,  ctrl_block, image_A, image_B, running_image, boot_cfg, user_cfg */
    {
     0x400000,
     {0x0, (256 - 16) * SIZE_1K},
     {(256 - 16) * SIZE_1K, 16 * SIZE_1K},
     {256 * SIZE_1K, 768 * SIZE_1K},
     {1024 * SIZE_1K, 768 * SIZE_1K},
     {1792 * SIZE_1K, 2048 * SIZE_1K},
     {3840 * SIZE_1K, 16 * SIZE_1K},
     {(3840 + 16) * SIZE_1K, 240 * SIZE_1K}
     },

    {
     0x200000,
     {0x0, (128 - 16) * SIZE_1K},
     {(128 - 16) * SIZE_1K, 16 * SIZE_1K},
     {128 * SIZE_1K, 320 * SIZE_1K},
     {(128 + 320) * SIZE_1K, 320 * SIZE_1K},
     {768 * SIZE_1K, 1024 * SIZE_1K},
     {1792 * SIZE_1K, 16 * SIZE_1K},
     {(1792 + 16) * SIZE_1K, 240 * SIZE_1K}
     }
};

#define FLASH_PARTITION_ARRAY_CNT (sizeof(partition_info)/sizeof(board_partition_info_t))

board_partition_info_t global_partition_info = { 0 };


void music_spi_init_function()
{
    unsigned long clock_value = 0;
    /*When write a non-zero value to this register selects "GPIO" mode 
       for the SPI controller. In this mode, the other sw registers are visible.
       When a zero value is written to this register, reads to the SPI space are
       translated in reads of the external SPI flash devic */
    music_reg_wr(MUSIC_SPI_FS, 1);
    //clock_value |= 0x0;
    //clock_value |= spi_clock_div;

    /*clear remap */
    clock_value = 0x44;
    //clock_value |= spi_clock_div;
    music_reg_wr(MUSIC_SPI_CLOCK, clock_value);
    music_reg_wr(MUSIC_SPI_FS, 0);
}

#if 1
void
//music_spi_sector_erase(uint32_t addr)
music_spi_sector_erase()
{
    int rd = 1;
    uint32_t addr;

    addr = flash_write_addr;
    /* spi write enable */
    music_reg_wr(MUSIC_SPI_FS, 1);
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS);

#if 1
    music_spi_bit_banger(MUSIC_SPI_CMD_WREN);
    music_spi_go();

    music_spi_bit_banger(MUSIC_SPI_CMD_SECTOR_ERASE);
    music_spi_send_addr(addr);
    music_spi_go();

    /* polling whether operation complete */
    do
    {
        music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS);
        music_spi_bit_banger(MUSIC_SPI_CMD_RD_STATUS);
        music_spi_delay_8();
        rd = (music_reg_rd(MUSIC_SPI_RD_STATUS) & 1);
    }
    while (rd);

#endif
    music_reg_wr(MUSIC_SPI_FS, 0);
}

void
//music_spi_write_page(uint32_t addr, uint8_t *data, int len)
music_spi_write_page()
{
    int i;
    char ch;
    int rd = 1;
    uint8_t *data;

    uint32_t addr = flash_write_addr;
    int len = flash_write_len;
    data = flash_write_data;

    /* spi write enable */
    music_reg_wr(MUSIC_SPI_FS, 1);
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS);
    music_spi_bit_banger(MUSIC_SPI_CMD_WREN);
    music_spi_go();

    /*pp(page program) cmd */
    music_spi_bit_banger(MUSIC_SPI_CMD_PAGE_PROG);
    /*3 bytes address */
    music_spi_send_addr(addr);

    if (len % 4)
        len += (4 - (len % 4));
    for (i = 0; i < len; i++)
    {
        ch = *(data + (i / 4) * 4 + 3 - (i % 4));
        /*at least 1 byte data */
        music_spi_bit_banger(ch);
    }

    music_spi_go();
    /* polling whether operation complete */
    do
    {
        music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS);
        music_spi_bit_banger(MUSIC_SPI_CMD_RD_STATUS);
        music_spi_delay_8();
        rd = (music_reg_rd(MUSIC_SPI_RD_STATUS) & 1);
    }
    while (rd);

    //i = 1000000;
    //while(i--);
    music_reg_wr(MUSIC_SPI_FS, 0);
    //i = 1000000;
    //while(i--);
}

void mips_cache_flush();
#if 1
void spiflash_sector_erase(uint32_t addr)
{
    int i;
    uint32_t *func_p_from, *func_p_to;
    uint32_t spi_sector_erase_function[0x900 / 4];
    flash_write_addr = addr;
    spi_sector_erase_function_p = (void *)spi_sector_erase_function;
    func_p_to = (uint32_t *) spi_sector_erase_function;
    func_p_from = (uint32_t *) music_spi_sector_erase;
    for (i = 0; i < 0x900 / 4; i++)
        *func_p_to++ = *func_p_from++;
    //mips_cache_flush();
    //spi_sector_erase_function_p(addr);
    spi_sector_erase_function_p();
}

void spiflash_write_page(uint32_t addr, uint8_t * data, int len)
{
    int i;
    uint32_t *func_p_from, *func_p_to;
    uint32_t spi_write_page_function[0x900 / 4];

    flash_write_addr = addr;
    flash_write_data = data;
    flash_write_len = len;
    spi_write_page_function_p = (void *)spi_write_page_function;
    func_p_to = (uint32_t *) spi_write_page_function;
    func_p_from = (uint32_t *) music_spi_write_page;
    for (i = 0; i < 0x900 / 4; i++)
        *func_p_to++ = *func_p_from++;
    //mips_cache_flush();
    spi_write_page_function_p();
}
#else
//uint32_t spi_sector_erase_function[0x280/4];
//uint32_t spi_write_page_function[0x320/4];
uint32_t spi_sector_erase_function[0x900 / 4];
uint32_t spi_write_page_function[0x900 / 4];

void spi_function_init()
{
    int i;
    uint32_t *func_p_from, *func_p_to;
    //spi_sector_erase_function_p = (void *)(((uint32_t)spi_sector_erase_function)&0x1fffffff | 0x80000000);
    //printf("spi_sector_erase_function_p@0x%x\n\r", (uint32_t)spi_sector_erase_function_p);
    spi_sector_erase_function_p = (void *)spi_sector_erase_function;
    func_p_to = (uint32_t *) spi_sector_erase_function;
    func_p_from = (uint32_t *) music_spi_sector_erase;
    for (i = 0; i < 0x900 / 4; i++)
        *func_p_to++ = *func_p_from++;

    //spi_write_page_function_p = (void *)(((uint32_t)spi_write_page_function&0x1fffffff) | 0x80000000);
    spi_write_page_function_p = (void *)spi_write_page_function;
    func_p_to = (uint32_t *) spi_write_page_function;
    func_p_from = (uint32_t *) music_spi_write_page;
    for (i = 0; i < 0x900 / 4; i++)
        *func_p_to++ = *func_p_from++;

}

void spiflash_sector_erase(uint32_t addr)
{
    flash_write_addr = addr;
    //mips_cache_flush();
    //printf("start spi_sector_erase_function_p\n\r");
    //spi_sector_erase_function_p(addr);
    spi_sector_erase_function_p();
    //printf("end spi_sector_erase_function_p\n\r");
}

void spiflash_write_page(uint32_t addr, uint8_t * data, int len)
{
    flash_write_addr = addr;
    flash_write_data = data;
    flash_write_len = len;
    //mips_cache_flush();
    spi_write_page_function_p();
}
#endif
#endif

void spiflash_write_configure(uint32_t addr, uint8_t * data, int len)
{
    flash_write_addr = addr;
    flash_write_data = data;
    flash_write_len = len;
}

#if 0
void music_spi_init()
{
    int i;
    char *func_p_from, *func_p_to;
    char spi_init_function[0x100];
    spi_init_function_p = (void *)spi_init_function;
    func_p_to = (char *)spi_init_function_p;
    func_p_from = (char *)music_spi_init_function;
    for (i = 0; i < 0x100; i++)
        *func_p_to++ = *func_p_from++;
    spi_init_function_p();
}
#else
//char test[0x400];
void music_spi_init()
{
    int i;
    uint32_t *func_p_from, *func_p_to;
    uint32_t spi_init_function[0x100];
    printf("music_spi_init\n\r");
    spi_init_function_p = (void *)spi_init_function;
    func_p_to = (uint32_t *) spi_init_function_p;
    func_p_from = (uint32_t *) music_spi_init_function;
    for (i = 0; i < 0x100; i++)
        *func_p_to++ = *func_p_from++;
    //mips_cache_flush();
    spi_init_function_p();

    //spi_function_init();
#if 0
    printf("erase sector:0x%x\n", 0xb00000);
    spiflash_sector_erase(0xb00000);
#if 0
    for (i = 0; i < 0x400; i++)
        test[i] = i;
    test_p = test;
#endif

#if 1
    test_p = (uint8_t *) 0xbd002000;
    printf("write page\n\r");
    spiflash_write_page(0xb00000, test_p, 256);

    printf("compare sector\n\r");
#endif
    test_r = (uint32_t *) 0xb9b00000;
    test_q = (uint32_t *) 0xbd002000;

    for (i = 0; i < 0x40; i++)
    {
        if (*test_r != *test_q)
            printf("offset:0x%x real:0x%x != expect:0x%x\n", i, *test_r, *test_q);
        test_r++;
        test_q++;
    }
#endif
}

void music_upgrade_elite()
{
    uint32_t addr;
    uint8_t *from, *source;
    int i, nsect, total = 0, len_this_lp, bytes_this_page;
    int start_sect = 0;
    int len = 0x100000;
    uint8_t date_buf[MUSIC_SPI_PAGE_SIZE];
    uint32_t dst = 0;
    uint8_t *src;

    //from = (uint8_t *)0xb9100000;
    //to = (uint8_t *)0xb92800000;
    addr = 0x280000;
    source = (uint8_t *) 0xb9100000;
    //source = (uint8_t *)0xa0800000;

    nsect = len / MUSIC_SPI_SECTOR_SIZE;
    if (len % MUSIC_SPI_SECTOR_SIZE)
    {
        nsect++;
    }
    start_sect = addr / MUSIC_SPI_SECTOR_SIZE;

    printf("erase flash\n\r");
    for (i = start_sect; i < start_sect + nsect; i++)
    {
        spiflash_sector_erase(i * MUSIC_SPI_SECTOR_SIZE);
    }

    printf("write image to flash\n\r");
    while (total < len)
    {
        src = source + total;
        dst = addr + total;
        bytes_this_page = MUSIC_SPI_PAGE_SIZE - (addr % MUSIC_SPI_PAGE_SIZE);
        len_this_lp = ((len - total) > bytes_this_page) ? bytes_this_page : (len - total);
#if 0
        if (total % (MUSIC_SPI_PAGE_SIZE * 0x80) == 0)
            printf("src:%x dst:%x len_this_lp:%x total:%x len:%x\n", src, dst | 0xb9000000, len_this_lp, total, len);
#endif
        from = src;
        for (i = 0; i < len_this_lp; i++)
        {
            date_buf[i] = *from++;
        }
        spiflash_write_page(dst, date_buf, len_this_lp);
        total += len_this_lp;
    }

#if 0
    from = (uint8_t *) 0xb9c00000;
    to = (uint8_t *) 0xb9b00000;
    for (i = 0; i < len; i++)
    {
        if (*from != *to)
            printf("offset:0x%x real:0x%x != expect:0x%x\n", i, *to, *from);
        from++;
        to++;
    }
#endif

    printf("go into upgradable elite image!\n\r");
    spi_init_function_p = (void *)0xb9300000;
    spi_init_function_p();
    //spiflash_sector_erase(0xb00000);
    //spiflash_write_page(0xb00000, test_p, 256);
}
#endif

void music_spi_read_id()
{
    uint32_t flash_id;
    music_reg_wr(MUSIC_SPI_FS, 1);
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS);
    music_spi_bit_banger(MUSIC_SPI_CMD_RDID);
    music_spi_delay_8();
    music_spi_delay_8();
    music_spi_delay_8();

    flash_write_addr = music_reg_rd(MUSIC_SPI_RD_STATUS) & 0xffffff;
    music_reg_wr(MUSIC_SPI_FS, 0);
    return;
}

uint32_t spiflash_spi_read_id()
{
    int i;
    uint32_t *func_p_from, *func_p_to;
    uint32_t spi_write_page_function[0x300 / 4];

    flash_write_addr = 0;
    spi_common_function_p = (void *)spi_write_page_function;
    func_p_to = (uint32_t *) spi_write_page_function;
    func_p_from = (uint32_t *) music_spi_read_id;
    for (i = 0; i < 0x300 / 4; i++)
        *func_p_to++ = *func_p_from++;
    spi_common_function_p();
    return flash_write_addr;
}

void board_get_partition_info()
{
    //board_partition_info_t partition_info
    uint32_t flash_id = 0;
    uint32_t flash_size = 0, i = 0;

    flash_id = spiflash_spi_read_id();
    for (i = 0; i < SPI_FLASH_ARRAY_CNT; i++)
    {
        if (spi_flash_array[i].flash_id == flash_id)
        {
            flash_size = spi_flash_array[i].flash_size;
            printf("flash id:0x%x, size:0x%x Bytes\n", flash_id, flash_size);
            break;
        }
    }

    if (i == SPI_FLASH_ARRAY_CNT)
    {
        printf("flash id:0x%x, using default size:0x%x Bytes\n", flash_id, DEFAULT_FLASH_SIZE);
        flash_size = DEFAULT_FLASH_SIZE;

    }

    for (i = 0; i < FLASH_PARTITION_ARRAY_CNT; i++)
    {
        if (flash_size == partition_info[i].flash_size)
            memcpy(&global_partition_info, &(partition_info[i]), sizeof(board_partition_info_t));
    }

    global_flash_size = flash_size;
    global_image_a_offset = global_partition_info.image_a.segment_addr;
    global_image_b_offset = global_partition_info.image_b.segment_addr;
    global_running_offset = global_partition_info.running_image.segment_addr;
    global_ctrl_block_offset = global_partition_info.control_block.segment_addr;
    global_boot_cfg_offset = global_partition_info.boot_cfg.segment_addr;
    global_user_cfg_offset = global_partition_info.user_cfg.segment_addr;
    return;
}

void partition_info_show(void)
{
    printf("flash partition info:\n\r");
    printf("[partition total size]:0x%x\n", global_partition_info.flash_size);
    printf("[loader] addr:0x%x size:0x%x\n", global_partition_info.loader.segment_addr,
           global_partition_info.loader.segment_size);
    printf("[control_block] addr:0x%x size:0x%x\n", global_partition_info.control_block.segment_addr,
           global_partition_info.control_block.segment_size);
    printf("[image_a] addr:0x%x size:0x%x\n", global_partition_info.image_a.segment_addr,
           global_partition_info.image_a.segment_size);
    printf("[image_b] addr:0x%x size:0x%x\n", global_partition_info.image_b.segment_addr,
           global_partition_info.image_b.segment_size);
    printf("[running_image] addr:0x%x size:0x%x\n", global_partition_info.running_image.segment_addr,
           global_partition_info.running_image.segment_size);
    printf("[boot_cfg] addr:0x%x size:0x%x\n", global_partition_info.boot_cfg.segment_addr,
           global_partition_info.boot_cfg.segment_size);
    printf("[user_cfg] addr:0x%x size:0x%x\n", global_partition_info.user_cfg.segment_addr,
           global_partition_info.user_cfg.segment_size);
}
