//#include <stdio.h>
#include "zlib.h"
#include "music_spi.h"
#include "music_upgrade.h"

int printf(const char *fmt, ...);

#define MALLOC_MEM_SIZE 0xa000
#define PER_OUT_LEN 0x100

char private_mem[MALLOC_MEM_SIZE];
char *private_mem_p = private_mem;  //0xdd005000;//private_mem;
unsigned char decompress_out_buf[PER_OUT_LEN] = { 0 };
unsigned int malloced_mem_size = 0;

voidpf zcalloc(voidpf opaque, unsigned items, unsigned size)
{
#define	ZALLOC_ALIGNMENT	16
    void *p;

    size *= items;
    size = (size + ZALLOC_ALIGNMENT - 1) & ~(ZALLOC_ALIGNMENT - 1);
    //printf("malloced_mem_size:0x%x\n\r", malloced_mem_size);
    if (malloced_mem_size + size < MALLOC_MEM_SIZE)
    {
        p = private_mem_p;
        private_mem_p += size;
        malloced_mem_size += size;
    }
    else
    {
        //p = 0;
        //p = (void *)0xb8150000;
        p = (void *)0xdd008000;
    }
    //printf("zcaloc: addr:0x%x size:0x%x\n", p, size);
#ifdef TODO
    p = malloc(size);
#endif

    return (p);
}

void zcfree(voidpf opaque, voidpf ptr)
{
#ifdef TODO
    free(ptr);
#endif
}

int zlib_init(z_stream * zstream, unsigned char *in_buf, unsigned char *out_buf)
{
    zstream->zalloc = zcalloc;
    zstream->zfree = zcfree;
    zstream->opaque = Z_NULL;
    zstream->next_in = in_buf;
    zstream->avail_in = 0;
    zstream->next_out = out_buf;

//#define ZLIB_WINDOW_BITS      15
//#define ZLIB_ENABLE_ZLIB_GZIP 32 
#define ZLIB_WINDOW_BITS      15
#define ZLIB_ENABLE_ZLIB_GZIP 32
    int status = inflateInit2(zstream, (ZLIB_WINDOW_BITS | ZLIB_ENABLE_ZLIB_GZIP));
    if (status < 0)
    {
        printf("inflateInit2 fail %d\n", status);
        return -1;
    }
    return 0;
}

void zlib_exit(z_stream * zstream)
{
    inflateEnd(zstream);
}
unsigned int zlib_gunzip(z_stream * zstream, unsigned int total_len, unsigned char *out_buf, int per_out_len)
{
    int status, this_len;

    zstream->avail_in = total_len;
    zstream->avail_out = per_out_len;
    zstream->next_out = out_buf;

    status = inflate(zstream, Z_NO_FLUSH);
    if (status < 0)
    {
        printf("inflate fail %d\n", status);
        return 0;
    }
    this_len = per_out_len - zstream->avail_out;

    return this_len;
}

int uncompress_image(uint32_t decompress_addr, uint32_t compress_addr)
{
#ifdef TODO
    FILE *in_file = fopen("vmlinux.bin.gz", "rb");
    FILE *out_file = fopen("vmlinux.bin", "wb");
#endif
    //unsigned char in_buf[0x400000] = {0};//must be greater than file size
    unsigned char *in_buf = (unsigned char *)(0x99000000 + compress_addr + IMAGE_HEADER_SIZE);
    //unsigned char decompress_out_buf[PER_OUT_LEN]= {0};
    int per_out_len = PER_OUT_LEN;
    int bytes_read = 300000;    //0x80000;
    z_stream zstream = { 0 };
    uint32_t wirte_addr = decompress_addr;
    printf("decompress_addr:0x%x compress_addr:0x%x\n", decompress_addr, compress_addr);

    if (zlib_init(&zstream, in_buf, decompress_out_buf) != 0)
    {
        return -1;
    }

#ifdef TODO
    int bytes_read = fread(in_buf, sizeof(char), sizeof(in_buf), in_file);
    if (bytes_read == 0)
    {
        zlib_exit(&zstream);
        printf("read file error\n");
        return -1;
    }
#endif

    while (1)
    {
        //printf("line: %d\n\r", __LINE__);
        int this_len = zlib_gunzip(&zstream, bytes_read, decompress_out_buf, per_out_len);
        {
            if (wirte_addr % 0x1000 == 0)
            {
                //printf("spiflash_sector_erase addr:0x%x\n\r", wirte_addr);
                spiflash_sector_erase(wirte_addr);
            }
            //printf("spiflash_write_page addr:0x%x\n\r", wirte_addr);
            spiflash_write_page(wirte_addr, decompress_out_buf, 0x100);
            wirte_addr += 0x100;
        }
        //printf("line: %d this_len:0x%x\n\r", __LINE__, this_len);
        {
            //printf("%08x %08x\n\r", decompress_out_buf[0], decompress_out_buf[1]);
        }
        //fwrite (out_buf, sizeof (char), this_len, out_file);

        if (this_len < per_out_len)
        {
            break;
        }
    }

    zlib_exit(&zstream);
#ifdef TODO
    fclose(in_file);
    fclose(out_file);
#endif

    return 0;
}
