#include <common.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <config.h>
#include "music_soc.h"
#include "music_phy.h"

uint32_t switch_reg_rd(uint32_t reg)
{
    return music_reg_rd(SWITCH_REG_BASE + reg);
}

int32_t switch_reg_wr(uint32_t reg, uint32_t val)
{
    music_reg_wr((SWITCH_REG_BASE + (reg)), val);

    return 0;
}

uint32_t
_switch_mdio_phy_rd(uint32_t phy_id, uint32_t port_id,
                    uint32_t reg, uint32_t * val, uint32_t(*reg_rd) (uint32_t), int32_t(*reg_wr) (uint32_t, uint32_t))
{
    uint32_t data = 0, cnt = 10;

    data |= ((phy_id * PHY_ACCEL_PORT_MAX + port_id % PHY_ACCEL_PORT_MAX)
             << MDIO_CTRL_PHY_ADDR_OFFSET) & MDIO_CTRL_PHY_ADDR_MASK;
    data |= (reg << MDIO_CTRL_REG_ADDR_OFFSET) & MDIO_CTRL_REG_ADDR_MASK;
    data |= (MDIO_CMD_READ << MDIO_CTRL_MDIO_CMD_OFFSET) & MDIO_CTRL_MDIO_CMD_MASK;
    data |= (phy_id << MDIO_CTRL_PHY_SEL_OFFSET) & MDIO_CTRL_PHY_SEL_MASK;
    data |= (1 << MDIO_CTRL_MDIO_BUSY_OFFSET) & MDIO_CTRL_MDIO_BUSY_MASK;

    reg_wr(REG_MDIO_CTRL, data);

    while ((reg_rd(REG_MDIO_CTRL) & MDIO_CTRL_MDIO_BUSY_MASK) && (cnt--))
    {
        udelay(1000);
    }

    *val = ((reg_rd(REG_MDIO_CTRL) & MDIO_CTRL_MDIO_DATA_MASK) >> MDIO_CTRL_MDIO_DATA_OFFSET);
    return 0;
}

uint32_t
_switch_mdio_phy_wr(uint32_t phy_id, uint32_t port_id,
                    uint8_t reg, uint32_t val, uint32_t(*reg_rd) (uint32_t), int32_t(*reg_wr) (uint32_t, uint32_t))
{
    uint32_t data = 0, cnt = 10;

    data |= ((phy_id * PHY_ACCEL_PORT_MAX + port_id % PHY_ACCEL_PORT_MAX)
             << MDIO_CTRL_PHY_ADDR_OFFSET) & MDIO_CTRL_PHY_ADDR_MASK;
    data |= (reg << MDIO_CTRL_REG_ADDR_OFFSET) & MDIO_CTRL_REG_ADDR_MASK;
    data |= (MDIO_CMD_WRITE << MDIO_CTRL_MDIO_CMD_OFFSET) & MDIO_CTRL_MDIO_CMD_MASK;
    data |= (phy_id << MDIO_CTRL_PHY_SEL_OFFSET) & MDIO_CTRL_PHY_SEL_MASK;
    data |= (val << MDIO_CTRL_MDIO_DATA_OFFSET) & MDIO_CTRL_MDIO_DATA_MASK;
    data |= (1 << MDIO_CTRL_MDIO_BUSY_OFFSET) & MDIO_CTRL_MDIO_BUSY_MASK;

    reg_wr(REG_MDIO_CTRL, data);

    while ((reg_rd(REG_MDIO_CTRL) & MDIO_CTRL_MDIO_BUSY_MASK) && (cnt--))
    {
        udelay(1000);
    }

    return 0;
}

static void music_switch_phy_dbg_reg_wr(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr, uint32_t phy_val)
{
    switch_mdio_phy_wr(phy_id, port_id, 0x1D, phy_addr);
    switch_mdio_phy_wr(phy_id, port_id, 0x1E, phy_val);
}

static void music_switch_phy_mmd3_reg_wr(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr, uint32_t phy_val)
{
    switch_mdio_phy_wr(phy_id, port_id, 0xD, 0x0003);
    switch_mdio_phy_wr(phy_id, port_id, 0xE, phy_addr);
    switch_mdio_phy_wr(phy_id, port_id, 0xD, 0x4003);
    switch_mdio_phy_wr(phy_id, port_id, 0xE, phy_val);
}

static uint32_t music_switch_phy_mmd3_reg_rd(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr)
{
    uint32_t val;

    switch_mdio_phy_wr(phy_id, port_id, 0xD, 0x0003);
    switch_mdio_phy_wr(phy_id, port_id, 0xE, phy_addr);
    switch_mdio_phy_wr(phy_id, port_id, 0xD, 0x4003);
    switch_mdio_phy_rd(phy_id, port_id, 0xE, &val);

    return val;
}

static void music_switch_phy_mmd7_reg_wr(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr, uint32_t phy_val)
{
    switch_mdio_phy_wr(phy_id, port_id, 0xD, 0x0007);
    switch_mdio_phy_wr(phy_id, port_id, 0xE, phy_addr);
    switch_mdio_phy_wr(phy_id, port_id, 0xD, 0x4007);
    switch_mdio_phy_wr(phy_id, port_id, 0xE, phy_val);
}

static void music_switch_phy_id_get(uint32_t phy_id, uint32_t * phy_id0, uint32_t * phy_id1)
{
    uint32_t port_id = 0;

    switch_mdio_phy_rd(phy_id, port_id, PHY_REG_PHY_ID0, phy_id0);
    switch_mdio_phy_rd(phy_id, port_id, PHY_REG_PHY_ID1, phy_id1);
}

static void music_switch_b0_phy_init(uint32_t phy_id)
{
    uint32_t port_id;

    for (port_id = 0; port_id < PHY_ACCEL_PORT_MAX; port_id++)
    {
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x8008, 0x3045);
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x800D, 0x4b3f);
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x805C, 0x312a);
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x800A, 0x1080);
        music_switch_phy_dbg_reg_wr(phy_id, port_id, 0x0, 0x82ea);
        music_switch_phy_mmd7_reg_wr(phy_id, port_id, 0x8010, 0xc07c);
    }
}

static void music_switch_a0_phy_init(uint32_t phy_id)
{
    uint32_t port_id;
    uint32_t val;

    for (port_id = 0; port_id < PHY_ACCEL_PORT_MAX; port_id++)
    {
        music_switch_phy_mmd7_reg_wr(phy_id, port_id, 0x801a, 0x59aa);
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x806e, 0x888b);

        music_switch_phy_dbg_reg_wr(phy_id, port_id, 0x37, 0x6000);
        music_switch_phy_dbg_reg_wr(phy_id, port_id, 0x0, 0x82ee);
        music_switch_phy_dbg_reg_wr(phy_id, port_id, 0x5, 0x0c07);

        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x805b, 0x1917);

        val = music_switch_phy_mmd3_reg_rd(phy_id, port_id, 0x805D);
        val &= (~(1 << 8));
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x805D, val);

        val = music_switch_phy_mmd3_reg_rd(phy_id, port_id, 0x800D);
        val &= (~(1 << 15));
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x800D, val);

        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x8008, 0x101d);
        music_switch_phy_mmd3_reg_wr(phy_id, port_id, 0x800A, 0x1080);
    }
}

void music_switch_phy_init(void)
{
    uint32_t phy_id;
    uint32_t phy_id0, phy_id1;

    for (phy_id = 0; phy_id < PHY_ACCEL_MAX; phy_id++)
    {
        music_switch_phy_id_get(phy_id, &phy_id0, &phy_id1);

        if (MUSIC_PHY_IS_ACCEL_A0(phy_id0, phy_id1))
        {
            music_switch_a0_phy_init(phy_id);

        }
        else if (MUSIC_PHY_IS_ACCEL_B0(phy_id0, phy_id1))
        {
            music_switch_b0_phy_init(phy_id);
        }
    }
}

#ifdef MUSIC_B2B_SUPPORT
static void music_switch_slave_phy_dbg_reg_wr(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr, uint32_t phy_val)
{
    switch_slave_mdio_phy_wr(phy_id, port_id, 0x1D, phy_addr);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0x1E, phy_val);
}

static void music_switch_slave_phy_mmd3_reg_wr(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr, uint32_t phy_val)
{
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xD, 0x0003);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xE, phy_addr);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xD, 0x4003);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xE, phy_val);
}

static uint32_t music_switch_slave_phy_mmd3_reg_rd(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr)
{
    uint32_t val;

    switch_slave_mdio_phy_wr(phy_id, port_id, 0xD, 0x0003);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xE, phy_addr);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xD, 0x4003);
    switch_slave_mdio_phy_rd(phy_id, port_id, 0xE, &val);

    return val;
}

static void music_switch_slave_phy_mmd7_reg_wr(uint32_t phy_id, uint32_t port_id, uint32_t phy_addr, uint32_t phy_val)
{
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xD, 0x0007);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xE, phy_addr);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xD, 0x4007);
    switch_slave_mdio_phy_wr(phy_id, port_id, 0xE, phy_val);
}

static void music_switch_slave_phy_id_get(uint32_t phy_id, uint32_t * phy_id0, uint32_t * phy_id1)
{
    uint32_t port_id = 0;

    switch_slave_mdio_phy_rd(phy_id, port_id, PHY_REG_PHY_ID0, phy_id0);
    switch_slave_mdio_phy_rd(phy_id, port_id, PHY_REG_PHY_ID1, phy_id1);
}

static void music_switch_slave_b0_phy_init(uint32_t phy_id)
{
    uint32_t port_id;

    for (port_id = 0; port_id < PHY_ACCEL_PORT_MAX; port_id++)
    {
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x8008, 0x3045);
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x800D, 0x4b3f);
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x805C, 0x312a);
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x800A, 0x1080);
        music_switch_slave_phy_dbg_reg_wr(phy_id, port_id, 0x0, 0x82ea);
        music_switch_slave_phy_mmd7_reg_wr(phy_id, port_id, 0x8010, 0xc07c);
    }
}

static void music_switch_slave_a0_phy_init(uint32_t phy_id)
{
    uint32_t port_id;
    uint32_t val;

    for (port_id = 0; port_id < PHY_ACCEL_PORT_MAX; port_id++)
    {
        music_switch_slave_phy_mmd7_reg_wr(phy_id, port_id, 0x801a, 0x59aa);
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x806e, 0x888b);

        music_switch_slave_phy_dbg_reg_wr(phy_id, port_id, 0x37, 0x6000);
        music_switch_slave_phy_dbg_reg_wr(phy_id, port_id, 0x0, 0x82ee);
        music_switch_slave_phy_dbg_reg_wr(phy_id, port_id, 0x5, 0x0c07);

        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x805b, 0x1917);

        val = music_switch_slave_phy_mmd3_reg_rd(phy_id, port_id, 0x805D);
        val &= (~(1 << 8));
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x805D, val);

        val = music_switch_slave_phy_mmd3_reg_rd(phy_id, port_id, 0x800D);
        val &= (~(1 << 15));
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x800D, val);

        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x8008, 0x101d);
        music_switch_slave_phy_mmd3_reg_wr(phy_id, port_id, 0x800A, 0x1080);
    }
}

void music_switch_slave_phy_init(void)
{
    uint32_t phy_id;
    uint32_t phy_id0, phy_id1;

    for (phy_id = 0; phy_id < PHY_ACCEL_MAX; phy_id++)
    {
        music_switch_slave_phy_id_get(phy_id, &phy_id0, &phy_id1);

        if (MUSIC_PHY_IS_ACCEL_A0(phy_id0, phy_id1))
        {
            music_switch_slave_a0_phy_init(phy_id);

        }
        else if (MUSIC_PHY_IS_ACCEL_B0(phy_id0, phy_id1))
        {
            music_switch_slave_b0_phy_init(phy_id);
        }
    }
}
#endif

uint32_t music_miiphy_read(char *devname, unsigned char phaddr, unsigned char reg, unsigned short *val)
{
    uint32_t phy_id = phaddr / PHY_ACCEL_PORT_MAX;
    uint32_t port_id = phaddr % PHY_ACCEL_PORT_MAX;
    uint32_t temp = 0;

#if !defined(PHY_PORT_START) && !defined(PHY_PORT_SLAVE_START)
    switch_mdio_phy_rd(phy_id, port_id, reg, &temp);
    *val = (unsigned short)temp;
    return 0;

#else

#ifdef PHY_PORT_START
    if ((phaddr >= PHY_PORT_START) && (phaddr <= PHY_PORT_END))
    {
        switch_mdio_phy_rd(phy_id, port_id, reg, &temp);
    }
#endif
#ifdef PHY_PORT_SLAVE_START
    if ((phaddr >= PHY_PORT_SLAVE_START) && (phaddr <= PHY_PORT_SLAVE_END))
    {
        phy_id = (phaddr - PHY_PORT_SLAVE_START) / PHY_ACCEL_PORT_MAX;
        port_id = (phaddr - PHY_PORT_SLAVE_START) % PHY_ACCEL_PORT_MAX;
        switch_slave_mdio_phy_rd(phy_id, port_id, reg, &temp);
    }
    *val = (unsigned short)temp;
    return 0;
#endif

#endif
}

uint32_t music_miiphy_write(char *devname, unsigned char phaddr, unsigned char reg, unsigned short val)
{
    uint32_t phy_id = phaddr / PHY_ACCEL_PORT_MAX;
    uint32_t port_id = phaddr % PHY_ACCEL_PORT_MAX;

#if !defined(PHY_PORT_START) && !defined(PHY_PORT_SLAVE_START)
    switch_mdio_phy_wr(phy_id, port_id, reg, val);
    return 0;
#else

#ifdef PHY_PORT_START
    if ((phaddr >= PHY_PORT_START) && (phaddr <= PHY_PORT_END))
    {
        switch_mdio_phy_wr(phy_id, port_id, reg, val);
    }
#endif
#ifdef PHY_PORT_SLAVE_START
    if ((phaddr >= PHY_PORT_SLAVE_START) && (phaddr <= PHY_PORT_SLAVE_END))
    {
        phy_id = (phaddr - PHY_PORT_SLAVE_START) / PHY_ACCEL_PORT_MAX;
        port_id = (phaddr - PHY_PORT_SLAVE_START) % PHY_ACCEL_PORT_MAX;
        switch_slave_mdio_phy_wr(phy_id, port_id, reg, val);
    }
#endif
    return 0;

#endif
}
