#include <common.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <config.h>
#include "music_soc.h"
#include "music_switch.h"


#define switch_reg_wr(addr, val)  music_reg_wr(MUSIC_SWITCH_BASE + addr, val)
#define switch_reg_rd(addr)       music_reg_rd(MUSIC_SWITCH_BASE + addr)
uint32_t mdio_switch_reg_rd(uint32_t addr);
int32_t mdio_switch_reg_wr(uint32_t addr, uint32_t data);
void music_switch_phy_init(void);
void music_switch_slave_phy_init(void);

static void music_switch_phy_reset_enable(void)
{
    uint32_t reg_addr = REG_MAC_XMII_CTRL, reg_val;

    /*clear phy reset */
    reg_val = switch_reg_rd(reg_addr);
    reg_val |= (MAC_XMII_PHY_RST_EN_CFG_MASK);
    switch_reg_wr(reg_addr, reg_val);
}

static void music_switch_phy_reset_release(void)
{
    uint32_t reg_addr = REG_MAC_XMII_CTRL, reg_val;

    /*clear phy reset */
    reg_val = switch_reg_rd(reg_addr);
    reg_val &= (~MAC_XMII_PHY_RST_EN_CFG_MASK);
    switch_reg_wr(reg_addr, reg_val);
}

static void music_switch_mac_enable(void)
{
    uint32_t reg_addr;
    uint32_t port;

    for (port = 0; port < 24; port++)
    {
        reg_addr = REG_PORT_STATUS + port * 0x100;
        switch_reg_wr(reg_addr, 0x000012c2);
    }
}

static void music_switch_mac_disable(void)
{
    uint32_t reg_addr;
    uint32_t port;

    for (port = 0; port < 24; port++)
    {
        reg_addr = REG_PORT_STATUS + port * 0x100;
        switch_reg_wr(reg_addr, 0);
    }
}

static void music_switch_mac_func(void)
{
    switch_reg_wr(0x1c, 0x01fffffe);
    switch_reg_wr(0x15010, 0x10ffffff);
}

void music_switch_soft_reset(void)
{
    uint32_t reg_addr = REG_GLOBAL_CTRL_0, reg_val;

    reg_val = switch_reg_rd(reg_addr);
    reg_val |= (GLOBAL_CTRL_0_SOFT_RST_MASK | GLOBAL_CTRL_0_REG_RST_EN_MASK);
    switch_reg_wr(reg_addr, reg_val);

    printf("switch reset done\n\r");
}

void music_switch_main(void)
{
    music_switch_soft_reset();
    music_switch_mac_disable();
    music_switch_phy_reset_release();
    music_switch_phy_init();

    /* use PHY polling mode disable for MAC by default
     * music_switch_mac_func();
     */

    music_switch_mac_enable();

    printf("sw init done\n\r");
}


#ifdef MUSIC_B2B_SUPPORT
static void music_switch_slave_phy_reset_enable(void)
{
    uint32_t reg_addr = REG_MAC_XMII_CTRL, reg_val;

    /*clear phy reset */
    reg_val = mdio_switch_reg_rd(reg_addr);
    reg_val |= (MAC_XMII_PHY_RST_EN_CFG_MASK);
    mdio_switch_reg_wr(reg_addr, reg_val);
}

static void music_switch_slave_phy_reset_release(void)
{
    uint32_t reg_addr = REG_MAC_XMII_CTRL, reg_val;

    /*clear phy reset */
    reg_val = mdio_switch_reg_rd(reg_addr);
    reg_val &= (~MAC_XMII_PHY_RST_EN_CFG_MASK);
    mdio_switch_reg_wr(reg_addr, reg_val);
}

static void music_switch_slave_mac_enable(void)
{
    uint32_t reg_addr;
    uint32_t port;

    for (port = 0; port < 24; port++)
    {
        reg_addr = REG_PORT_STATUS + port * 0x100;
        mdio_switch_reg_wr(reg_addr, 0x000012c2);
    }
}

static void music_switch_slave_mac_disable(void)
{
    uint32_t reg_addr;
    uint32_t port;

    for (port = 0; port < 24; port++)
    {
        reg_addr = REG_PORT_STATUS + port * 0x100;
        mdio_switch_reg_wr(reg_addr, 0);
    }
}

static void music_switch_slave_mac_func(void)
{
    mdio_switch_reg_wr(0x1c, 0x01fffffe);
    mdio_switch_reg_wr(0x15010, 0x10ffffff);
}

void music_switch_slave_soft_reset(void)
{
    uint32_t reg_addr = REG_GLOBAL_CTRL_0, reg_val;

    reg_val = mdio_switch_reg_rd(reg_addr);
    reg_val |= (GLOBAL_CTRL_0_SOFT_RST_MASK | GLOBAL_CTRL_0_REG_RST_EN_MASK);
    mdio_switch_reg_wr(reg_addr, reg_val);

    printf("switch slave reset done\n\r");
}

void music_phy_48p_main(void)
{
    //music_switch_soft_reset();

    music_switch_slave_soft_reset();
    music_switch_mac_disable();
    music_switch_slave_mac_disable();

    music_switch_phy_reset_release();
    music_switch_slave_phy_reset_release();

    music_switch_phy_init();
    music_switch_slave_phy_init();

    /* use PHY polling mode disable for MAC by default
       music_switch_mac_func();
       music_switch_slave_mac_func();
     */

    music_switch_mac_enable();
    music_switch_slave_mac_enable();

    printf("sw 48p init done\n\r");
}
#endif
