/* 
 * Copyright (c) 2012 Qualcomm Atheros, Inc.. 
 * All Rights Reserved. 
 * Qualcomm Atheros Confidential and Proprietary. 
 *Notifications and licenses are retained for attribution purposes only */
/*
 * Copyright 2001 MontaVista Software Inc.
 * Author: Jun Sun, jsun@mvista.com or jsun@junsun.net
 * Copyright (C) 2000, 2001 Ralf Baechle (ralf@gnu.org)
 *
 * Copyright (C) 2007 Lemote Inc. & Insititute of Computing Technology
 * Author: Fuxin Zhang, zhangfx@lemote.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "addrspace.h"
#include <stdarg.h>

#define vivo_reg_rd(_phys)    (*(volatile unsigned int *)KSEG1ADDR(_phys))
#define vivo_reg_wr_nf(_phys, _val) \
                    ((*(volatile unsigned int *)KSEG1ADDR(_phys)) = (_val))

#define vivo_reg_wr(_phys, _val) do {     \
                    vivo_reg_wr_nf(_phys, _val);  \
                    vivo_reg_rd(_phys);       \
}while(0);



#define CONFIG_BAUDRATE 115200

#define ATH_S124_UART1_BASE  0x18060000
#define ATH_S124_UART2_BASE  0x18070000
#define AR_UART_BASE         ATH_S124_UART1_BASE

#define	REG_OFFSET		     4
/* register offset */
#define OFS_RCV_BUFFER		(0*REG_OFFSET)
#define OFS_TRANS_HOLD		(0*REG_OFFSET)
#define OFS_SEND_BUFFER		(0*REG_OFFSET)
#define OFS_INTR_ENABLE		(1*REG_OFFSET)
#define OFS_INTR_ID		    (2*REG_OFFSET)
#define OFS_FIFO_CONTROL    (2*REG_OFFSET)
#define OFS_DATA_FORMAT		(3*REG_OFFSET)
#define OFS_LINE_CONTROL	(3*REG_OFFSET)
#define OFS_MODEM_CONTROL	(4*REG_OFFSET)
#define OFS_RS232_OUTPUT	(4*REG_OFFSET)
#define OFS_LINE_STATUS		(5*REG_OFFSET)
#define OFS_MODEM_STATUS	(6*REG_OFFSET)
#define OFS_RS232_INPUT		(6*REG_OFFSET)
#define OFS_SCRATCH_PAD		(7*REG_OFFSET)

#define OFS_DIVISOR_LSB		(0*REG_OFFSET)
#define OFS_DIVISOR_MSB		(1*REG_OFFSET)



#define UART16550_READ(y)	    vivo_reg_rd((AR_UART_BASE+(y)))
#define UART16550_WRITE(x, z)	vivo_reg_wr((AR_UART_BASE+(x)), (z))

#define REG_EFUSE_HIGH_VAL                      0x10
#define REG_SWITCH_EFUSE                        REG_EFUSE_HIGH_VAL
#define SWITCH_EFUSE_CPU_SPEED_MODE_MASK        0x00000007
#define REG_SWITCH_BASE                         0x18800000

int vsprintf(char *buf, const char *fmt, va_list args);

int soc_bus_freq_get(void)
{
    /*000=200MHz, 001=300MHz, 010=400MHz, 011=480MHz, 
       100=533MHz, 101=600MHz. 110 and 111 Reserved. */

    unsigned long cpu_speed_mhz[8] = { 200, 300, 400, 480, 533, 600, 300, 300 };
    unsigned long reg_val = vivo_reg_rd(REG_SWITCH_BASE + REG_SWITCH_EFUSE) & SWITCH_EFUSE_CPU_SPEED_MODE_MASK;

    return ((cpu_speed_mhz[reg_val] * 1000 * 1000) / 2);
}

void serial_puts(const char *s);

int serial_init(void)
{
    int div = 0;
    int val = 0x07;

    div = soc_bus_freq_get() / (16 * CONFIG_BAUDRATE);

    /* 
     * Enable FIFO mode 
     */
    /* enable FIFO */
    //val = UART16550_READ(OFS_FIFO_CONTROL);
    UART16550_WRITE(OFS_FIFO_CONTROL, (val | 0x01));

    /* reset transmit FIFO and recieve FIFO */
    //val = UART16550_READ(OFS_FIFO_CONTROL);
    UART16550_WRITE(OFS_FIFO_CONTROL, (val | 0x06));
    /*set DIAB bit */
    UART16550_WRITE(OFS_LINE_CONTROL, 0x80);

    /* set divisor */
    UART16550_WRITE(OFS_DIVISOR_LSB, (div & 0xff));
    UART16550_WRITE(OFS_DIVISOR_MSB, ((div >> 8) & 0xff));

    /* clear DIAB bit */
    UART16550_WRITE(OFS_LINE_CONTROL, 0x00);

    /* set data format */
    UART16550_WRITE(OFS_DATA_FORMAT, 0x3);

    UART16550_WRITE(OFS_INTR_ENABLE, 0);

    serial_puts("serial_init done\n");
    return 0;
}

int serial_tstc(void)
{
    return (UART16550_READ(OFS_LINE_STATUS) & 0x1);
}

unsigned int serial_getc(void)
{
    while (!serial_tstc());

    return UART16550_READ(OFS_RCV_BUFFER);
}

void serial_putc(unsigned int byte)
{
    if (byte == '\n')
        serial_putc('\r');

    while (((UART16550_READ(OFS_LINE_STATUS)) & 0x20) == 0x0);
    UART16550_WRITE(OFS_SEND_BUFFER, byte);
}

void serial_setbrg(void)
{
}

void serial_puts(const char *s)
{
    while (*s)
    {
        serial_putc(*s++);
    }
}
int printf(const char *fmt, ...)
{
    va_list args;
    unsigned long i;
    char printbuffer[512];

    va_start(args, fmt);

    /* For this to work, printbuffer must be larger than
     * anything we ever want to print.
     */
    i = vsprintf(printbuffer, fmt, args);
    va_end(args);

    /* Print the string */
    //puts (printbuffer);
    serial_puts(printbuffer);

    return 0;
}
